/**
 * \file gre/dlgviewzoom.h
 * \brief Dialog to zoom/pan view to specified location/scale
 *
 * \if NODOC
 * $Id: dlgviewzoom.h_v 1.12 2013/02/15 20:42:21 mju Exp $
 *
 * $Log: dlgviewzoom.h_v $
 * Revision 1.12  2013/02/15 20:42:21  mju
 * Fix dox hdr.
 *
 * Revision 1.11  2012/11/08 18:59:20  mju
 * Use coordopzoned and spatref hdrs.
 *
 * Revision 1.10  2010/07/20 16:34:31  mju
 * Change getCenter.
 *
 * Revision 1.9  2010/06/07 14:11:48  mju
 * Add typedef for thisclass.
 *
 * Revision 1.8  2007/10/01 20:44:25  mju
 * More view_legacy.
 *
 * Revision 1.7  2005/10/17 20:32:37  mju
 * Exclude log comments from docs.
 *
 * Revision 1.6  2005/10/14 17:30:00  mju
 * GRE namespace.
 *
 * Revision 1.5  2005/06/24 14:58:18  mju
 * Don't use mgui templates.
 *
 * Revision 1.4  2005/06/23 21:56:01  mju
 * Exclude private section from doxygen.
 *
 * Revision 1.3  2004/07/12 16:00:04  mju
 * Use coordop_zoned.
 *
 * Revision 1.2  2004/07/09 17:40:57  mju
 * Support spatref.
 *
 * Revision 1.1  2003/12/22 20:21:09  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_DLGVIEWZOOM_H
#define	INC_GRE_DLGVIEWZOOM_H

#ifndef INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef INC_MGUI_FORMBTXT_H
#include <mgui/formbtxt.h>
#endif

#ifndef INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef INC_MI32_COORDOPZONED_H
#include <mi32/coordopzoned.h>
#endif

#ifndef INC_MI32_SPATREF_H
#include <mi32/spatref.h>
#endif


namespace GRE {
class VIEW;

class DLG_VIEWZOOM : public MGUI::DLGSHELL {
	public:
		DLG_VIEWZOOM (
			GRE::VIEW *view,
			const SPATREF::COORDREFSYS& crs
			);
		~DLG_VIEWZOOM (
			);
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_VIEWZOOM THISCLASS;

		GRE::VIEW *m_view;
		SPATREF::COORDREFSYS m_CoordRefSys;
		SPATREF::COORDOP_ZONED m_CoordOpZoned;
		MGUI::FORM_COMBOBOX m_CbxGroup;
		MGUI::FORM_COMBOBOX m_CbxZoomTo;
		MGUI::FORM_EDIT_NUMBER m_EditZoom;
		MGUI::CTRL_COMBOBOX m_CbxZoomUnits;
		MGUI::CTRL_GROUPBOX m_GbxCenter;
		MGUI::FORM_BUTTONTEXT m_BtxCenterRefSys;
		MGUI::FORM_EDIT_STRING m_EditCenterMGRS;
		MGUI::FORM_EDIT_NUMBER m_EditCenterY;
		MGUI::FORM_EDIT_NUMBER m_EditCenterX;

		void CheckEnableApply ();
		int GetCenter (DPOINT2D& center, bool CheckInGroup = false);
		void OnCmdRefSys ();
		void OnComboZoomTo ();
		void OnComboZoomUnits ();
		void UpdateControls ();
		void UpdateTitle ();
		void UpdateZoomControls ();
		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnApply ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}

#endif
