/**
 * \file gre/lrmapgrd.h
 * \brief GRE::LAYER_MAPGRID class definitions.
 *
 * \if NODOC
 * $Id: lrmapgrd.h_v 1.40 2013/02/15 20:45:44 mju Exp $
 *
 * $Log: lrmapgrd.h_v $
 * Revision 1.40  2013/02/15 20:45:44  mju
 * Fix dox hdr.
 *
 * Revision 1.39  2012/06/13 15:04:58  mju
 * Remove v_appendPopupMenuItems.
 *
 * Revision 1.38  2011/10/20 13:19:52  mju
 * Remove redundant hdr.
 *
 * Revision 1.37  2011/05/10 13:47:46  mju
 * Add m_gridFlags to dispparm for mapgriddispflag values so don't overlap common flags.
 *
 * Revision 1.36  2010/10/18 13:10:52  mju
 * Add v_snapPoint/IsAllowed.
 *
 * Revision 1.35  2010/02/02 21:52:19  mju
 * Change tick to include location on both grid and map rectangles.
 *
 * Revision 1.34  2010/02/02 18:10:41  mju
 * Add tickFindEdgePoint.
 *
 * Revision 1.33  2009/04/21 17:24:20  mju
 * Add flag_ExcludeFromGroupExtents.
 *
 * Revision 1.32  2008/01/07 18:52:03  mju
 * Add dispflag_noFullCoordLowest.
 *
 * Revision 1.31  2008/01/02 22:45:05  mju
 * Add menuAddViewPopupItems.
 *
 * Revision 1.30  2007/10/03 17:34:52  mju
 * Add dispflag_matchGroupExtents.
 *
 * Revision 1.29  2007/08/02 21:27:43  dwilliss
 * added method to get SML class members
 *
 * Revision 1.28  2007/05/02 13:31:17  mju
 * Add latlon fmt for DDMM.
 *
 * Revision 1.27  2006/09/28 13:14:26  mju
 * Use elemstyl hdr.
 *
 * Revision 1.26  2006/09/15 15:30:34  mju
 * Serializer changes.
 *
 * Revision 1.25  2005/12/08 22:40:37  mju
 * Remove mem24 virtual.
 *
 * Revision 1.24  2005/10/19 20:34:03  mju
 * Use gre namespace instead of legacy names.
 *
 * Revision 1.23  2005/10/14 17:31:27  mju
 * GRE namespace and treeview.
 *
 * Revision 1.22  2005/04/21 19:48:28  mju
 * Add xXX and xXXN formatting.
 *
 * Revision 1.21  2005/03/31 16:57:06  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.20  2005/02/25 23:35:34  scowan
 * Added copy virtual method implementation.
 *
 * Revision 1.19  2005/01/20 16:56:51  mju
 * Redesign layer type registration.
 *
 * Revision 1.18  2004/10/15 21:26:19  ldyas
 * added needsmem24 method
 *
 * Revision 1.17  2004/10/15 14:36:15  ldyas
 * *** empty log message ***
 *
 * Revision 1.16  2004/10/14 22:02:13  ldyas
 * isantialiased now nonconst.
 *
 * Revision 1.15  2004/10/06 14:23:02  ldyas
 * *** empty log message ***
 *
 * Revision 1.14  2004/10/01 22:38:57  ldyas
 * added new mapgriddispflags for antialiasing and thin line width hinting.
 *
 * Revision 1.13  2004/06/28 17:53:48  mju
 * Convert to spatref system.
 *
 * Revision 1.12  2004/03/05 18:06:10  mju
 * Use virtual dtor if have virtual fns.
 *
 * Revision 1.11  2003/09/15 13:48:59  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.10  2003/07/30 15:24:23  mju
 * Ignore private sections.
 *
 * Revision 1.9  2003/03/10 16:09:52  mju
 * Save toggled tick coords.
 *
 * Revision 1.8  2003/03/10 14:34:57  mju
 * Use ifdef for appendpopup.
 *
 * Revision 1.7  2003/03/07 23:20:25  mju
 * Add ability to toggle tick colors.
 * Add more UTM label formats.
 *
 * Revision 1.6  2002/11/04 21:01:03  mju
 * Add separate control for corner coords in 2D/3D.
 *
 * Revision 1.5  2002/10/15 22:01:42  mju
 * Add more text formatting options.
 * Add option to show corner labels.
 * Add field for border text offset.
 *
 * Revision 1.4  2002/09/23 17:20:53  mju
 * Add InitExtents to DISPPARM.
 *
 * Revision 1.3  2002/09/20 17:03:19  mju
 * Only declare v_ControlPanel if X or MFC.
 *
 * Revision 1.1  2002/09/10 14:24:51  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LRMAPGRD_H
#define	INC_GRE_LRMAPGRD_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif


namespace GRE {
//===================================================================================================================

//! GRE layer class for viewing Map Grid.
class LAYER_MAPGRID : public GRE::LAYER {
	public:

		class DISPPARM : public GRE::LAYER::DISPPARM {
			public:

				class DLG;										//!< Layer settings dialog, defined in <gre/uimapgrd.h>

				TEXTSTYLE CoordLabelStyle;
				TEXTSTYLE AxisLabelStyle;
				double xinterval;					//!< Interval in meters for non lat/lon projections
				double yinterval;
				double zinterval;
				double xdegrees;					//!< Interval to use for lat/lon (in degrees)
				double ydegrees;
				SPATREF::COORDREFSYS IntervalCRS;	//!< CRS of grid interval
				SPATREF::COORDREFSYS ExtentsCRS;		//!< CRS of grid extents
				DRECT3D MapExtents;				//!< Extents (sans ticks/text) in ExtentsProj
				UINT8 IntervalUnitsXY;			//!< X-Y interval units (if not lat-lon)
				UINT8 IntervalUnitsZ;			//!< Z interval units
				UINT8 IntervalLatLonFormat;	//!< Format for interval lat/lon prompts
				UINT8 ExtentsUnits; 				//!< Units for entering extents if not lat-lon
				UINT8 ExtentsLatLonFormat;		//!< Format for extents lat/lon prompts
				LINESTYLE StyleGrid;				//!< Style for grid lines
				LINESTYLE StyleBorder;			//!< Style for border/neatline
				LINESTYLE StyleBorderTickExt;	//!< Style for ticks outside border
				LINESTYLE StyleInteriorTick;	//!< Style for interior ticks
				COLOR BorderTickIntColor1;		//!< Primary color for border tick interior
				COLOR BorderTickIntColor2;		//!< Alternate color for border tick interior
				COLOR InteriorTickColor2;		//!< Alternate color for interior ticks
				double GridWidth;					//!< Thickness of grid lines
				double BorderWidth;				//!< Thickness of border
				double BorderTickExtWidth;		//!< Thickness of exterior border ticks
				double BorderTickExtLength;	//!< Outside length of border ticks
				double BorderTickIntWidth;		//!< Thickness of interior border ticks
				double BorderTickIntLength;	//!< Inside length of border ticks
				double InteriorTickWidth;		//!< Thickness of interior ticks
				double InteriorTickLength;		//!< Size of interior ticks
				double BorderTextOffset;		//!< Offset to border text from edge/tick endpoint
				double GraphicMapScale;			//!< Scale for graphics
				UINT8 GridWidthUnits;
				UINT8 BorderWidthUnits;
				UINT8 BorderTickExtWidthUnits;
				UINT8 BorderTickExtLengthUnits;
				UINT8 BorderTickIntWidthUnits;
				UINT8 BorderTickIntLengthUnits;
				UINT8 InteriorTickLengthUnits;
				UINT8 InteriorTickWidthUnits;
				UINT8 BorderTextOffsetUnits;
				UINT32 CoordFormatLatLon;		//!< Coordinate format for lat-lon
				UINT32 CoordFormatProj;			//!< Coordinate format for projection other than lat/lon
				UINT8 OldProjUnits;				//!< For old map grids used in conversion
				UINT8 VerticalAxes;
				UINT8 CoordOrient;				//!< Coordinate orientation
				MIUNICODE CoordUnitNameXY[32];
				MIUNICODE CoordUnitNameZ[32];
				MIUNICODE AxisLabelX[64];
				MIUNICODE AxisLabelY[64];
				MIUNICODE AxisLabelZ[64];
				DOUBLE_ARRAY<DPOINT2D> m_ToggledTickCoords;	//!< Coordinates of toggled ticks in 'extents' projection
				UINT32 m_GridFlags;									//!< MAPGRIDDISPFLAG values

				DISPPARM ();
				DISPPARM (const DISPPARM& rhs) { Copy(rhs); }
				virtual ~DISPPARM () { Free(); }

				DISPPARM& operator= (
					const DISPPARM& rhs
					) { if (this != &rhs) { Free(); Copy(rhs); } return (*this); }

				//! Initialize extents based on current projection and layer's group extents.
				//! The specified layer's group will be used to determine the extents,
				//! ignoring the layer itself.
				void InitExtents (
					const LAYER_MAPGRID *layer
					);
				//! Validate dispparm, adjusting if necessary.
				virtual ERRVALUE Validate ();

			protected:
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;
				static int SerializerCB_TickToggledCoords (SERIALIZER& serializer, const SERIALIZERITEM *itemdef, void *cbdata, SERIALIZERITEM::ACTION action);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				void Copy (const DISPPARM&);
				void Free ();
				#endif //!< GENERATING_DOXYGEN_OUTPUT
			};

		//! Register this layer type
		static void RegisterType (
			);

		//! Install SML class, called by internal SML function.
		static void SMLClassInstall (
			SML::CONTEXT *context
			);

		static const SML::CLASSMEMBER* GetSMLClassMembers (
			);

		//! Construct layer and insert into list
		explicit LAYER_MAPGRID (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			GRE::LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		//! Destructor.
		virtual ~LAYER_MAPGRID ();

		//! Retrieve layer-specific display parameters.
		const DISPPARM& GetDispParm (
			) const { return (m_dispparm); }

		//! Set layer-specific display parameters.
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef LAYER_MAPGRID THISCLASS;

		class PRIVDRAWDATA;
		friend class PRIVDRAWDATA;

		struct TICK {
			enum LOCATION {
				LOCATION_Interior =		0,
				LOCATION_MinX =			1,
				LOCATION_MaxX =			2,
				LOCATION_MinY =			3,
				LOCATION_MaxY =			4,
				LOCATION_Other =			5
				};
			enum FLAGS {
				FLAG_None =			0x00,
				FLAG_Lowest =		0x01,
				FLAG_AltColor =	0x02,
				FLAG_Corner =		0x04
				};
			DPOINT2D m_pt;						//!< Coordinates in "extents" projection
			DPOINT2D m_pt2;					//!< Secondary point for determining orientation
			double m_label;					//!< X or Y map coordinate for tick point label
			LOCATION m_LocGridRect;
			LOCATION m_LocMapRect;
			FLAGS m_flags;

			TICK (
				const DPOINT2D& pt,
				const DPOINT2D& pt2,
				LOCATION LocGridRect,
				LOCATION LocMapRect,
				FLAGS flags = FLAG_None,
				double label = 0
				);
			};

		TRANS2D_MAPGEN m_TransMapToExt;
		DRECT2D m_GridRect;
		DPOINT2D m_GridInterval;
		SIMPLE_ARRAY<TICK> m_Ticks;
		int m_ActiveTick;
		void *m_LabelList;

		static void SMLGET_MapGridInterval (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SMLARG*);
		static void SMLSET_MapGridInterval (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SMLARG*);

	#ifdef USE_MGUI
		void OnMenuToggleTickColor (MGUI::ID id, bool ToggleOn);
	#endif

		ERRVALUE OnToggleTickColor ();
		void ReadDefaultsIni ();
		void SaveDefaultsIni ();
		ERRVALUE TickAddCornerPoints ();
		ERRVALUE TickAddEdgePoint (const DPOINT2D& label, const DPOINT2D& pt, const DPOINT2D& pt2, TICK::LOCATION type, TICK::FLAGS flags);
		ERRVALUE TickCalcPoints ();
		bool TickFindEdgePoint (const DPOINT2D& ptGridInside, const DPOINT2D& ptGridOutside, const DPOINT2D& ptMapInside, const DPOINT2D& ptMapOutside, TICK::LOCATION TickType, TICK::FLAGS TickFlags);
		ERRVALUE TickLoadToggledCoords ();
		ERRVALUE TickSaveToggledCoords ();

		// Overrides from GRE::LAYER
		virtual void v_ComputeMargins (double& tmargin, double& bmargin, double& lmargin, double& rmargin);
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent, UINT32 flags = 0);
	#endif
		virtual GRE::LAYER* v_Copy (GRE::GROUP *group) const;
		virtual ERRVALUE v_Draw (GRE::LAYERDC& layerdc);
		virtual bool v_IsAntialiased ()const;
		virtual bool v_IsLineWidthHinted () const;
	#ifdef USE_MGUI
		virtual void v_MenuAddViewPopupItems (MGUI::MENU& menu, const DPOINT2D *point, VIEW *view);
	#endif
		virtual const char* v_SerialGetTagName () const;
		virtual ERRVALUE v_SerialRead (SERIALIZER& serializer);
		virtual void v_SetAntialiased (bool antialias);
		virtual void v_SetDftName (bool ReplaceExisting);
		virtual void v_SetHintThinLines (bool hint);
		virtual int v_SnapPoint (DPOINT3D& ptObject, SNAPTO SnapTo, double MaxDistance, VIEW *view) const;
		virtual bool v_SnapPointIsAllowed (SNAPTO SnapTo) const;
		virtual int v_UpdateExtents (bool forceupdate);

		LAYER_MAPGRID (const LAYER_MAPGRID&);
		LAYER_MAPGRID& operator= (const LAYER_MAPGRID&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#define	MAPGRIDDISPFLAG_SizeRelative			(0x00000001)	//!< Sizes relative to TickScale
#define	MAPGRIDDISPFLAG_ShowBorderTicksExt	(0x00000002)	//!< Show tick marks outside border
#define	MAPGRIDDISPFLAG_ShowBorderTicksInt	(0x00000004)	//!< Show tick marks inside border
#define	MAPGRIDDISPFLAG_NoFullCoordLowest	(0x00000008)	//!< Don't force full coordinates on lowest X/Y value
#define	MAPGRIDDISPFLAG_ShowGrid				(0x00000010)	//!< Show grid
#define	MAPGRIDDISPFLAG_ShowBorderTicks		(0x00000020)	//!< Show tick marks at border (OLD)
#define	MAPGRIDDISPFLAG_ShowCoordText2D		(0x00000040)	//!< Show coordinate text in 2D
#define	MAPGRIDDISPFLAG_ShowBorder				(0x00000080)	//!< Show border
#define	MAPGRIDDISPFLAG_ShowInteriorTicks	(0x00000100)	//!< Show interior tick marks
#define	MAPGRIDDISPFLAG_ShowCoordText3D		(0x00000200)	//!< Show coordinate text in 3D
#define	MAPGRIDDISPFLAG_ShowBorderBottom3D	(0x00000400)	//!< Show border at Z = zmin in 3D
#define	MAPGRIDDISPFLAG_ShowBorderTop3D		(0x00000800)	//!< Show border at Z = zmax in 3D
#define	MAPGRIDDISPFLAG_BorderRelative		(0x00001000)	//!< Interval is relative to borders, not 0
#define	MAPGRIDDISPFLAG_ShowVerticalTicks	(0x00002000)	//!< Show ticks on vertical axes
#define	MAPGRIDDISPFLAG_LayoutScale			(0x00004000)	//!< Use layout mapscale
#define	MAPGRIDDISPFLAG_ShowVerticalScale	(0x00008000)	//!< Show scale on vertical axes
#define	MAPGRIDDISPFLAG_ShowMinSec0			(0x00010000)	//!< Show minutes/seconds in lat-lon even if 0
#define	MAPGRIDDISPFLAG_ShowDegOnlyIfMin0	(0x00020000)	//!< Show degrees in lat-lon only if minutes = 0
#define	MAPGRIDDISPFLAG_ShowCornerText2D		(0x00040000)	//!< Show coordinates at corners
#define	MAPGRIDDISPFLAG_ShowCornerText3D		(0x00080000)	//!< Show coordinates at corners
#define	MAPGRIDDISPFLAG_Antialias				(0x00100000)	//!< Antialias
#define	MAPGRIDDISPFLAG_HintThinLines			(0x00200000)	//!< Hint Thin Lines via Transparency
#define	MAPGRIDDISPFLAG_MatchGroupExtents	(0x00400000)	//!< Match group extents
#define	MAPGRIDDISPFLAG_ExcludeFromGroupExt	(0x00800000)	//!< Exclude from group extents

#define	MAPGRIDDISPFLAGS_ShowBorderTicksAny	(MAPGRIDDISPFLAG_ShowBorderTicks | MAPGRIDDISPFLAG_ShowBorderTicksExt | MAPGRIDDISPFLAG_ShowBorderTicksInt)

#define	MAPGRID_COORDFMTLATLON_MASK			0x000F
#define	MAPGRID_COORDFMTLATLON_DDMMSS			0x0000
#define	MAPGRID_COORDFMTLATLON_DDdddd			0x0001
#define	MAPGRID_COORDFMTLATLON_DDMMmmm		0x0002
#define	MAPGRID_COORDFMTLATLON_DDMMSSss		0x0003
#define	MAPGRID_COORDFMTLATLON_DDMMSSs		0x0004
#define	MAPGRID_COORDFMTLATLON_DDMM			0x0005
#define	MAPGRID_COORDFMTLATLON_None			0x000F
#define	MAPGRID_COORDFMTLATLON_NoSymbol		0x0020
#define	MAPGRID_COORDFMTLATLON_AddSpace		0x0040
#define	MAPGRID_COORDFMTLATLON_Leading0Deg	0x0080
#define	MAPGRID_COORDFMTLATLON_Leading0MS	0x0100
#define	MAPGRID_COORDFMTLATLON_DirFirst		0x0200
#define	MAPGRID_COORDFMTLATLON_DirLast		0x0400

#define  MAPGRID_COORDFMTPROJ_Space				0
#define  MAPGRID_COORDFMTPROJ_Comma				1
#define  MAPGRID_COORDFMTPROJ_Period			2
#define  MAPGRID_COORDFMTPROJ_NoSpace			3
#define  MAPGRID_COORDFMTPROJ_xXXxxxN			4
#define  MAPGRID_COORDFMTPROJ_xXXxxx			5
#define  MAPGRID_COORDFMTPROJ_xXXxxx_XX		6
#define  MAPGRID_COORDFMTPROJ_xXXxxxN_XX		7
#define  MAPGRID_COORDFMTPROJ_xXXxxxNp_XX		8
#define  MAPGRID_COORDFMTPROJ_xXXN				9
#define  MAPGRID_COORDFMTPROJ_xXX				10

#define	MAPGRID_COORDORIENT_Horizontal		0
#define	MAPGRID_COORDORIENT_Parallel			1
#define	MAPGRID_COORDORIENT_Perpendicular	2
#define	MAPGRID_COORDORIENT_AlongTick			3
#define	MAPGRID_COORDORIENT_Vertical			4

#define	MAPGRID_VERTICALAXES_None				0
#define	MAPGRID_VERTICALAXES_Farthest1		1
#define	MAPGRID_VERTICALAXES_Farthest2		2
#define	MAPGRID_VERTICALAXES_Farthest3		3
#define	MAPGRID_VERTICALAXES_All				4


//===================================================================================================================
}	// End namespace GRE


#endif	//!< INC_GRE_LRMAPGRD_H
