/**
 * \file gre/lrscaleb.h
 * \brief GRE_LAYER_SCALEBAR class definitions.
 *
 * \if NODOC
 * $Id: lrscaleb.h_v 1.12 2013/02/15 20:51:37 mju Exp $
 *
 * $Log: lrscaleb.h_v $
 * Revision 1.12  2013/02/15 20:51:37  mju
 * Fix dox hdr.
 *
 * Revision 1.11  2011/10/20 13:26:20  mju
 * Remove redundant hdr.
 *
 * Revision 1.10  2006/09/28 13:26:11  mju
 * Use elemstyl hdr.
 *
 * Revision 1.9  2006/09/15 15:30:47  mju
 * Serializer changes.
 *
 * Revision 1.8  2005/10/26 16:38:10  mju
 * Use gre namespace instead of legacy types.
 *
 * Revision 1.7  2005/10/14 17:31:49  mju
 * GRE namespace and treeview.
 *
 * Revision 1.6  2005/02/25 23:39:43  scowan
 * Added copy virtual method implementation.
 *
 * Revision 1.5  2005/01/20 16:57:19  mju
 * Redesign layer type registration.
 *
 * Revision 1.4  2004/03/05 21:29:15  mju
 * Make dtor virtual if virtual fns.
 *
 * Revision 1.3  2003/09/15 13:48:59  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2003/07/30 15:27:04  mju
 * Ignore private sections.
 *
 * Revision 1.1  2002/09/20 17:37:19  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LRSCALEB_H
#define	INC_GRE_LRSCALEB_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif


namespace GRE {
//===================================================================================================================
//! GRE "scale bar" layer.
class LAYER_SCALEBAR : public GRE::LAYER {
	public:

		class DISPPARM : public GRE::LAYER::DISPPARM {
			public:

				class DLG;

				double Length;					//!< length in whatever units user choose
				double BarWidth;				//!< bar thickness in users choice of units
				double Scale;					//!< scale for width/sizes, etc
				int MajorDiv;
				int MinorDiv;					//!< Number of major/minor divisions
				UINT8 LengthUnits;
				UINT8 BarWidthUnits;
				UINT8 LineWidthUnits;
				TEXTSTYLE TextStyle;
				LINESTYLE Style[4];			//!< Colors and such
				MISTRING UnitLabel;

				DISPPARM ();
				DISPPARM (const DISPPARM& rhs) {
					Copy(rhs);
					}
				virtual ~DISPPARM () {
					Free();
					}

				DISPPARM& operator= (const DISPPARM& rhs) {
					if (this != &rhs) {
						Free();
						Copy(rhs);
						}
					return (*this);
					}

			protected:
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				void Copy (const DISPPARM&);
				void Free ();
				#endif //!< GENERATING_DOXYGEN_OUTPUT
			};

		//! Register this layer type
		static void RegisterType (
			);

		//! Install SML class, called by internal SML function.
		static void SMLClassInstall (
			SMLCONTEXT *context
			);

		//! Construct layer and insert into list
		explicit LAYER_SCALEBAR (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			GRE::LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		//! Destructor.
		virtual ~LAYER_SCALEBAR ();

		#ifndef GENERATING_DOXYGEN_OUTPUT
		int DrawScaleBar (GRE::LAYERDC&, void*);
		#endif //!< GENERATING_DOXYGEN_OUTPUT

		//! Retrieve layer-specific display parameters.
		const DISPPARM& GetDispParm (
			) const { return (m_dispparm); }

		//! Set layer-specific display parameters.
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		void ReadDefaultsIni ();
		void SaveDefaultsIni ();

		//! Overrides from GRE::LAYER.
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent,UINT32 flags = 0);
	#endif
		virtual GRE::LAYER* v_Copy (GRE::GROUP *group) const;
		virtual ERRVALUE v_Draw (GRE::LAYERDC& layerdc);
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetDftName (bool ReplaceExisting);
		virtual int v_UpdateExtents (bool forceupdate);

		LAYER_SCALEBAR (const LAYER_SCALEBAR&);
		LAYER_SCALEBAR& operator= (const LAYER_SCALEBAR&);

		#endif //!< GENERATING_DOXYGEN_OUTPUT
	};

#define SCALEBARDISPFLAG_NegativePart	0x00000001
#define SCALEBARDISPFLAG_CenterLine		0x00000002
#define SCALEBARDISPFLAG_ShowTicks		0x00000004
#define SCALEBARDISPFLAG_TextOnBottom	0x00000008
#define SCALEBARDISPFLAG_ShowUnits		0x00000010
#define SCALEBARDISPFLAG_CenterUnits	0x00000020
#define SCALEBARDISPFLAG_LayoutScale	0x00000040

//===================================================================================================================
}	// End namespace GRE

#endif	//!< INC_GRE_LRSCALEB_H
