/**
 * \file gre/uidatatip.h
 * \brief GRE::DATATIPPARMS::FORM
 *
 * \if NODOC
 * $Id: uidatatip.h_v 1.12 2012/06/04 17:44:38 mju Exp $
 *
 * $Log: uidatatip.h_v $
 * Revision 1.12  2012/06/04 17:44:38  mju
 * Add create with objitem/elemtype.
 *
 * Revision 1.11  2011/11/28 17:22:19  mju
 * Fix dox dependencies.
 *
 * Revision 1.3  2006/03/24 16:17:15  mju
 * Add methods to get objitem/elemtype and set delegate.
 *
 * Revision 1.1  2005/10/14 17:33:09  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_UIDATATIP_H
#define	INC_GRE_UIDATATIP_H

#ifndef	INC_GRE_DATATIP_H
#include <gre/datatip.h>
#endif

#ifndef	INC_MGUI_FORM_H
#include <mgui/form.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class OBJITEM;
	}
#endif

namespace GRE {
//===================================================================================================================

//! Form for editing DATATIPPARMS.
class DATATIPPARMS::FORM : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		FORM ();

		//! Destructor.
		virtual ~FORM ();

		//! Create the form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane		//!< Parent pane
			);

		//! Create form with specific associated object.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const RVC::OBJITEM& ObjItem,				//!< Object item reference
			ELEMTYPE ElemType								//!< Element type
			);

		//! Get element type.
		ELEMTYPE GetElemType () const;

		//! Get object item.
		const RVC::OBJITEM& GetObjItem () const;

		//! Get current value settings.
		const DATATIPPARMS& GetValues () const;

		//! Set delegate to call when values changed.
		void SetDelegateOnChangeValues (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set object and element type DataTip is for.
		ERRVALUE SetObject (
			const RVC::OBJITEM& ObjItem,				//!< Object item reference
			ELEMTYPE ElemType								//!< Element type
			);

		//! Set new values.
		ERRVALUE SetValues (
			const DATATIPPARMS& values,				//!< New values to set
			bool notify = true							//!< Call OnChangeValues if 'true'
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

}	// End namespace GRE

#endif	// INC_GRE_UIDATATIP_H
