/** 
 * \file mgui/dlgschedulejob.h 
 * \brief Definitions for MGUI::DLG_SCHEDULE_JOB class.
 *
 * \if NODOC
 * $Id: dlgschedulejob.h_v 1.4 2011/11/22 13:27:52 mju Exp $
 *
 * $Log: dlgschedulejob.h_v $
 * Revision 1.4  2011/11/22 13:27:52  mju
 * Remove redundant hdrs.
 *
 * Revision 1.3  2011/08/26 13:28:50  mju
 * Use formradiobuttons hdr.
 *
 * Revision 1.2  2009/07/06 20:38:14  bcolombe
 * Added changes for advanced schedule options
 *
 * Revision 1.1  2009/06/22 14:55:25  bcolombe
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_DLG_SCHEDULE_JOB_H
#define	INC_MGUI_DLG_SCHEDULE_JOB_H

#ifndef INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef  INC_MGUI_FORMRADIOBUTTONS_H
#include <mgui/formradiobuttons.h>
#endif

#ifndef INC_MGUI_FORMCHECKLIST_H
#include <mgui/formchecklist.h>
#endif

#ifndef INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

class MIJOB;
namespace MGUI {

class DLG_SCHEDULE_JOB : public MGUI::DLGSHELL {
	public:
		DLG_SCHEDULE_JOB ();
		virtual ~DLG_SCHEDULE_JOB ();

		//! Display schedule dlg and write new job
		ERRVALUE CreateJob (
			MIJOB& job,						//! Normal job that will be given schedule parms and written
			MDLGPARENT DlgParent = 0		//! DlgParent for dialog
			);
 
		//! Initialize dlg from job
		void ReadJob (
			const FILEPATH& path			//! Path of job to read
			) {m_path = path;}

		//! Output dlg to existing job
		ERRVALUE WriteJob (
			const FILEPATH& path			//! Path of job to write
			) {return m_form.WriteToJob(path);}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class FORM_SCHEDULE_JOB : public MGUI::FORM_COMPOSITE {
			public:
				//! Constructor.
				FORM_SCHEDULE_JOB (
					);

				//! Destructor.
				virtual ~FORM_SCHEDULE_JOB (
					);

				//! Create form with label from resource lookup.
				void Create (
					MGUI::LAYOUT_PANE_BASE& ParentPane		//!< Parent pane
					);

				//! Set form values from schedule job
				ERRVALUE ReadFromJob (
					const FILEPATH& path	//!< path of job
					);

				//! Set values in schedule job from form
				ERRVALUE WriteToJob (
					const FILEPATH& path	//!< path of job
					);

			private:
				void OnTypeSelected();

				MGUI::LAYOUT_PANE m_DailyPane;
				MGUI::LAYOUT_PANE m_WeeklyPane;
				MGUI::LAYOUT_PANE m_MonthlyPane;

				MGUI::CTRL_EDIT_NUMBER m_StartTimeHour;
				MGUI::CTRL_EDIT_NUMBER m_StartTimeMin;
				MGUI::CTRL_COMBOBOX m_StartTimeAMPM;
				MGUI::FORM_COMBOBOX m_TypeCBox;

				MGUI::FORM_RADIOBUTTONS m_DailyRadio;
				MGUI::CTRL_EDIT_NUMBER m_EveryDays;
				MGUI::CTRL_TOGGLEBUTTON m_RepeatToggle;
				MGUI::CTRL_COMBOBOX m_RepeatCombo;
				MGUI::FORM_COMBOBOX m_RepeatDurationCombo;
				MGUI::CTRL_EDIT_NUMBER m_StartMonth;
				MGUI::CTRL_EDIT_NUMBER m_StartDay;
				MGUI::CTRL_EDIT_NUMBER m_StartYear;

				MGUI::CTRL_EDIT_NUMBER m_EveryWeeks;
				MGUI::FORM_CHECKLIST m_DayChecklist;

				MGUI::FORM_RADIOBUTTONS m_MonthlyRadio;
				MGUI::CTRL_COMBOBOX m_MonthlyCB1;
				MGUI::CTRL_COMBOBOX m_MonthlyCB2;
				MGUI::CTRL_EDIT_NUMBER m_MonthlyDay;
				MGUI::FORM_CHECKLIST m_MonthChecklist;				
			};

		virtual ERRVALUE v_CreateContent();
		FORM_SCHEDULE_JOB m_form;
		FILEPATH m_path;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}	// End of MGUI namespace

#endif		//	INC_MGUI_FORM_COORD_H
