/**
 * \file mgui/dlgselectbandnums.h
 * \brief Functions to prompt user to select band numbers from larger set
 *
 * \if NODOC
 * $Id: dlgselectbandnums.h_v 1.2 2011/11/22 13:29:14 mju Exp $
 *
 * $Log: dlgselectbandnums.h_v $
 * Revision 1.2  2011/11/22 13:29:14  mju
 * Remove redundant hdr.
 *
 * Revision 1.1  2009/04/30 15:03:45  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_DLGSELECTBANDNUMS_H
#define	INC_MGUI_DLGSELECTBANDNUMS_H

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

namespace MGUI {
class FORM_COMBOBOX;

//! Dialog to allow user to choose limited set of bands from larger set.
class DLG_SELECTBANDNUMS : public MGUI::DLGSHELL {
	public:

		enum FLAGS {
			FLAG_None =					0x00,
			FLAG_AllowNone =			0x01,		//!< Include "none", will return band number as -1
			FLAG_AllowDuplicate =	0x02,		//!< Allow same source band to be used more than once
			};

		//! Construct with band names from TEXTID array.
		DLG_SELECTBANDNUMS (
			const SIMPLE_ARRAY<INT16>& BandNums,	//!< Initial band numbers
			const TEXTID *BandNames,					//!< Band names, must at least as many entries as BandNums
			int NumSrcBands,								//!< Number of source bands to select from
			FLAGS flags = FLAG_None
			);

		virtual ~DLG_SELECTBANDNUMS ();

		//! Get selected band numbers.
		const SIMPLE_ARRAY<INT16> GetBandNums (
			) const { return (m_BandNums); }

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		typedef DLG_SELECTBANDNUMS THISCLASS;
		SIMPLE_ARRAY<INT16> m_BandNums;
		SIMPLE_ARRAY<TEXTID> m_BandNames;
		int m_NumSrcBands;
		FLAGS m_flags;
		MGUI::FORM_COMBOBOX *m_CbxBandNums;

		void OnCbxBandNum ();

		virtual ERRVALUE v_CreateContent ();
		#endif
	};

#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DLG_SELECTBANDNUMS::FLAGS)
#endif


}	// End namespace MGUI

#endif	// INC_MGUI_DLGSELECTBANDNUMS_H
