/**
 * \file mgui/toolarc.h
 * \brief MGUI::TOOL_ARC class definition
 *
 * \if NODOC
 * $Id: toolarc.h_v 1.8 2011/12/09 20:22:35 mju Exp $
 *
 * $Log: toolarc.h_v $
 * Revision 1.8  2011/12/09 20:22:35  mju
 * No context in v_draw.
 *
 * Revision 1.7  2011/06/21 14:02:28  mju
 * *** empty log message ***
 *
 * Revision 1.6  2008/02/15 23:36:47  scowan
 * Added manual entry stubs.
 *
 * Revision 1.5  2008/02/01 15:00:36  mju
 * Change button handlers to return bool.
 *
 * Revision 1.4  2008/01/04 18:58:44  mju
 * Generalize pointer button methods.
 *
 * Revision 1.1  2003/01/10 23:07:41  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_TOOLARC_H
#define	INC_MGUI_TOOLARC_H

#ifndef	INC_MGUI_TOOLBASE_H
#include <mgui/toolbase.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Circular arc tool
class TOOL_ARC : public TOOL_BASE {
	public:

		enum CLOSURE {
			CLOSURE_None,
			CLOSURE_Chord,
			CLOSURE_Wedge
			};

		//! Constructor.
		explicit TOOL_ARC (
			TOOLCANVAS *canvas
			);

		//! Destructor.
		virtual ~TOOL_ARC ();

		//! Get center position.
		const DPOINT2D& GetCenter (
			) const { return (m_center); }

		//! Get closure type.
		CLOSURE GetClosure (
			) const { return (m_closure); }

		//! Get radius.
		double GetRadius (
			) const { return (m_radius); }

		//! Get starting angle.
		//! @return Starting angle in radians.
		double GetStartAngle (
			) const { return (m_StartAngle); }

		//! Get sweep angle.
		//! @return Sweep angle in radians.
		double GetSweepAngle (
			) const { return (m_SweepAngle); }

		//! Set center of arc.
		void SetCenter (
			const DPOINT2D& newcenter
			);

		//! Set closure type.
		void SetClosure (
			CLOSURE closure
			);

		//! Set marker type and size.
		//! The default marker is MARKER_Plus with a size of 9 pixels.
		void SetMarker (
			MARKER marker,								//!< New marker type
			UINT16 size = 0							//!< New marker size, 0 to retain current size
			);

		void SetRadius (
			double newradius
			);

		void SetRadiusRange (
			double newminradius,
			double newmaxradius
			);

		//! Set starting angle.
		void SetStartAngle (
			double startangle							//!< Starting angle in radians
			);

		//! Set sweep angle
		void SetSweepAngle (
			double sweepangle							//!< Sweep angle in radians
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Enumeration constants
		enum ZONE {
			ZONE_NoFunction = 0,
			ZONE_Outside,
			ZONE_Inside,
			ZONE_Perimeter,
			ZONE_SetCenter,
			ZONE_Start,
			ZONE_End
			};
		//! Tool settings
		MARKER m_marker;
		UINT16 m_markersize;
		double m_minradius;						//!< Minimum radius
		double m_maxradius;						//!< Maximum radius, 0 if no maximum
		CLOSURE m_closure;
		//! Tool position values
		DPOINT2D m_center;						//!< Center
		double m_radius;
		double m_StartAngle;
		double m_SweepAngle;
		//! Values used during interaction
		ZONE m_CursorZone;						//!< Current cursor zone
		DPOINT2D m_prevpt;						//!< Previous point during button-down

		//! Manual Point Entry form forward declaration
		class MANUAL_ARC;

		//! Methods
		void AdjustTool (const DPOINT2D&, bool firsttime=false);
		void ComputeCursorZone (const DPOINT2D&);
		void StartTool (const DPOINT2D&);
		void UpdateCursor (const DPOINT2D&) const;

		//! TOOL_BASE overrides.
		virtual MANUAL_BASE* v_CreateManualBase ();
		virtual void v_DoDraw ();
		virtual void v_DoMove (double dx, double dy, ADJUSTFLAGS flags);
		virtual int v_GenerateRegion (REGION2D& region) const;
		virtual bool v_OnPointerButtonDown (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual void v_OnPointerMove (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_TOOLARC_H
