/**
 * \file anyrastv.h <mi32/anyrastv.h>
 * \brief ANYRASTVALUE definitions
 *
 * \if NODOC
 * $Id: anyrastv.h_v 1.8 2005/07/14 20:35:10 mju Exp $
 *
 * $Log: anyrastv.h_v $
 * Revision 1.8  2005/07/14 20:35:10  mju
 * Include memory hdr for memset.
 *
 * Revision 1.7  2004/03/22 15:04:53  mju
 * Include string.h for memset decl.
 *
 * Revision 1.6  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.5  2003/08/27 21:30:55  scowan
 * Added new celltype null value usage.
 *
 * Revision 1.4  2002/01/09 22:11:39  scowan
 * Added constructors and other meothds.
 *
 * Revision 1.3  2000/10/19 17:45:33  scowan
 * Checked in for vacation.
 *
 * Revision 1.2  1999/05/07  21:15:11  mju
 * Header restruct.
 *
 * Revision 1.1  1999/04/28  21:24:10  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_ANYRASTV_H
#define	INC_MI32_ANYRASTV_H

#ifndef	INC_MEMORY_H
#include <memory.h>		// Includes defn for memset()
#define	INC_MEMORY_H
#endif

#ifndef	INC_MI32_COMPLEX_H
#include <mi32/complex.h>
#endif

union ANYRASTVALUE {
	UINT8 ubyte;				//!<  all unsigned rasters <= 8 bit 
	INT8 byte;					//!<  signed 8 bit raster 
	UINT16 uint2;
	INT16 int2;
	UINT32 uint4;
	INT32 int4;
	FLOAT float4;
	DOUBLE float8;
	UINT8 RGB[3];
	UINT8 BGR[3];
	UINT8 CMYK[4];
	UINT8 KCMY[4];
	UINT8 RGBA[4];
	UINT8 ARGB[4];
	FCOMPLEXRI comp4;			//!<  Complex number values 
	DCOMPLEXRI comp8;
	FCOMPLEXMP comp4mp;
	DCOMPLEXMP comp8mp;
	UINT8 space[32];			//!<  For future expansion, 256/8 
	
	ANYRASTVALUE (
		) {
		memset(this, 0, sizeof(*this));
		}
		
	ANYRASTVALUE (
		const ANYRASTVALUE& rhs
		) {
		memcpy(this, &rhs, sizeof(*this));
		}
		
	ANYRASTVALUE& operator= (
		const ANYRASTVALUE& rhs
		) {
		if (this != &rhs) memcpy(this, &rhs, sizeof(*this));
		return (*this);
		}
		
	bool operator== (
		const ANYRASTVALUE& rhs
		) {
		return (memcmp(this, &rhs, sizeof(ANYRASTVALUE)) == 0);
		}
		
	bool operator!= (
		const ANYRASTVALUE& rhs
		) {
		return (memcmp(this, &rhs, sizeof(ANYRASTVALUE)) != 0);
		}
		
	void Clear (
		) {
		memset(this, 0, sizeof(*this));
		return;
		}
		
	};

#endif	//!<  INC_MI32_ANYRASTV_H 
