/**
 * \file gremode.h <mi32/gremode.h>
 * \brief Include file for GRE SELECT/STYLE MODE enums
 *
 * \if NODOC
 * $Id: gremode.h_v 1.9 2012/05/07 15:38:57 mju Exp $
 *
 * $Log: gremode.h_v $
 * Revision 1.9  2012/05/07 15:38:57  mju
 * Add bitwise ops.
 *
 * Revision 1.8  2010/05/10 17:37:58  mju
 * Add select/stylemode_Special/Special_MASK.
 *
 * Revision 1.2  2002/09/06 17:51:12  mju
 * Add STYLEMODE_NeedsTable.
 *
 * Revision 1.1  2002/04/12 14:14:16  scowan
 * Initial revision
 *
 * \endif
 **/

#ifndef INC_MI32_GREMODE_H
#define INC_MI32_GREMODE_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

 
enum SELECTMODE {
	SELECTMODE_None =					0x0000,
	SELECTMODE_All =					0x0001,
	SELECTMODE_ByElement =			0x0002,
	SELECTMODE_ByAttribute =		0x0004,
	SELECTMODE_ByQuery =				0x0010,
	SELECTMODE_Highlighted =		0x0020,	//!< Items specified in "HighlightSet" bit array
	SELECTMODE_ByMapScale =			0x0080,
	SELECTMODE_Special =				0x8000,
	SELECTMODE_Special_MASK =		0xF000,

	SELECTMODE_NeedsTable =			(SELECTMODE_ByAttribute | SELECTMODE_ByMapScale),
	SELECTMODE_NeedsDatabase =		(SELECTMODE_NeedsTable | SELECTMODE_ByQuery),
	};

enum STYLEMODE {
	STYLEMODE_AllSame =				0x0000,
	STYLEMODE_ByElement =			0x0002,
	STYLEMODE_ByAttribute =			0x0004,
	STYLEMODE_ByScript =				0x0010,
	STYLEMODE_ByTheme =				0x0040,
//	STYLEMODE_ByMapScale =			0x0080,
	STYLEMODE_ByThumbnailField =	0x0100,
	STYLEMODE_Special =				0x8000,
	STYLEMODE_Special_MASK =		0xF000,

	STYLEMODE_NeedsStyAsTable =	(STYLEMODE_ByAttribute | STYLEMODE_ByTheme),
	STYLEMODE_NeedsTable =			(STYLEMODE_NeedsStyAsTable),
	STYLEMODE_NeedsDatabase =		(STYLEMODE_NeedsTable | STYLEMODE_ByScript),
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(SELECTMODE)
DEFINE_ENUM_OP_BITWISE(STYLEMODE)
#endif

#endif
