/**
 * \file mi32/lidarlasrecordregion.h
 * \brief LIDAR::LAS::RECORD_REGION definitions
 *
 * \if NODOC
 * $Id: lidarlasrecordregion.h_v 1.2 2012/09/14 17:24:48 mju Exp $
 *
 * $Log: lidarlasrecordregion.h_v $
 * Revision 1.2  2012/09/14 17:24:48  mju
 * Make stuff used only while scanning private.
 * Add setCoordRefSys.
 *
 * Revision 1.1  2012/09/14 13:51:45  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_LIDARLASRECORDREGION_H
#define	INC_MI32_LIDARLASRECORDREGION_H

#ifndef	INC_MI32_LIDARLASRECORD_H
#include <mi32/lidarlasrecord.h>
#endif

#ifndef	INC_MI32_REGION2D_H
#include <mi32/region2d.h>
#endif


#ifdef RVCSYSDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class RECORDBUFFER_LSBFIRST;
#endif

namespace LIDAR {
namespace LAS {

//=====================================================================================================================

//! Bounding region record.
class CLASSLIBEXPORT RECORD_REGION : public RECORD_SCAN {
	public:

		//! Constructor.
		RECORD_REGION ();

		//! Destructor.
		virtual ~RECORD_REGION ();

		//! Get region.
		const REGION2D& GetRegion () const
			{ return (m_Region); }

		//! Set coordinate reference system on region.
		void SetCoordRefSys (
			const SPATREF::COORDREFSYS& crs			//!< CRS to set
			) { m_Region.SetCoordRefSys(crs); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		REGION2D m_Region;
		PRIV *m_pPriv;

		static ERRVALUE GetPolyLine (const RECORDBUFFER_LSBFIRST& buffer, UINT32& offset, POLYLINE& polyline);
		static ERRVALUE PutPolyLine (RECORDBUFFER_LSBFIRST& buffer, UINT32& offset, const POLYLINE& polyline);
		void Clear ();

		// RECORD overrides.
		virtual int v_Read (LAS::FILE& file);
		virtual ERRVALUE v_Write (LAS::FILE& file, bool ForceExtFile) const;
		// RECORD_SCAN overrides.
		virtual ERRVALUE v_AddPoint (UINT64 PointIdx, const POINTRECORD& PointRec);
		virtual ERRVALUE v_Finalize ();
		virtual ERRVALUE v_Initialize (LAS::FILE& file);

		#endif
	};

//=====================================================================================================================

}	// End namespace LAS
}	// End namespace LIDAR

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_LIDARLASRECORDREGION_H
