/*****************************************************************************
 *
 *	\file macres.h <mi32/macres.h>
 *	\brief Routines for reading Macintosh resource fork from a flat
 *		data file.
 *    This will only be useful on MacOS X where they kept the TrueType
 *		font "suitcase" idea but put it all in a data fork instead of a
 *		resource fork.  
 *
 *	\if NODOC
 *	$Log: macres.h_v $
 *	Revision 1.9  2006/02/16 16:12:46  dwilliss
 *	Changed an ifdef to use new define instead of deprecated
 *
 *	Revision 1.8  2006/01/24 21:51:07  dwilliss
 *	Allow to compile for C
 *
 *	Revision 1.7  2005/10/18 16:43:24  scowan
 *	*** empty log message ***
 *
 *	Revision 1.6  2005/10/18 16:20:16  scowan
 *	Changed function name.
 *
 *	Revision 1.5  2005/10/18 16:11:33  scowan
 *	Updated so that free type dll can call the one damn function it needs.
 *
 *	Revision 1.4  2005/10/13 16:46:01  scowan
 *	Convert to use file raw.
 *
 *	Revision 1.3  2005/07/14 16:18:23  mju
 *	Remove miodefns hdr.
 *
 *	Revision 1.2  2003/09/15 13:49:56  fileserver!dwilliss
 *	Doxygen
 *
 *	Revision 1.1  2001/11/07 23:52:07  dwilliss
 *	Initial revision
 *
 *		
 *	\endif
 ****************************************************************************/

#ifndef INC_MI32_MACRES_H
#define INC_MI32_MACRES_H

#ifndef INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifdef __cplusplus
class FILE_RAW;
#endif

#ifdef MISYSTEMDLL
	#define LIBEXPORT MI_DLLEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus	// Note: this gets included by freetype which ony compiles as C, not C++
//! Return the number of resources of a given type in a data-fork resource file
//!
//!	This is for MacOS X where Apple kept TrueType fonts in the resource file 
//! structure but in a flat file.
LIBEXPORT int MacNumResourcesOfType (
	FILE_RAW& file,		//!< Opened with FILE_RAW::Open()
	UINT32 type				//!< e.g. 'sfnt'
	);
#endif

#ifdef __cplusplus
//! Read the Nth resource of a given type from a data-fork resource file
//!
//!	This is for MacOS X where Apple kept TrueType fonts in the resource file 
//! structure but in a flat file.
//! If pData is NULL, just return the size of the buffer
//! If not NULL, *pData is allocated and should be MmFree()'d 
//! @return Size of the buffer allocated, 0 if not found, < 0 on error

LIBEXPORT int MacReadResourceByNum (
	FILE_RAW& file,		//!< Opened with FILE_RAW::Open()
	UINT32 type,		//!< e.g. 'sfnt'
	int num,				//!< Read the Nth resource (0-based)
	void **pData
	);
#endif

#ifdef __cplusplus
//! Read a resource of a given type and ID from a data-fork resource file
//!
//!	This is for MacOS X where Apple kept TrueType fonts in the resource file 
//! structure but in a flat file.
//! If pData is NULL, just return the size of the buffer
//! If not NULL, *pData is allocated and should be MmFree()'d 
//! @return Size of the buffer allocated, 0 if not found, < 0 on error

LIBEXPORT int MacReadResourceByID (
	FILE_RAW& file,		//!< Opened with FILE_RAW::Open()
	UINT32 type,		//!< e.g. 'sfnt'
	UINT16 ResID,		//!< Resource ID
	void **pData
	);
#endif

//! Read the Nth resource of a given type from a data-fork resource file
//!
//!	This is for MacOS X where Apple kept TrueType fonts in the resource file 
//! structure but in a flat file.
//! If pData is NULL, just return the size of the buffer
//! If not NULL, *pData is allocated and should be MmFree()'d 
//! @return Size of the buffer allocated, 0 if not found, < 0 on error

LIBEXPORT int MacReadResourceByNum2 (
	MIUNICODE* filename,		//!< Opened with FILE_RAW::Open()
	UINT32 type,		//!< e.g. 'sfnt'
	int num,				//!< Read the Nth resource (0-based)
	void **pData
	);

#ifdef __cplusplus
}
#endif

#endif
