/*****************************************************************************
 *
 * \file mi32/mg2cad.h
 * \brief 
 *
 * $Log: mg2cad.h_v $
 * Revision 1.6  2006/06/26 15:18:40  dwilliss
 * include mgd2.h
 *
 * Revision 1.5  2005/03/03 22:59:45  dwilliss
 * *** empty log message ***
 *
 * Revision 1.4  2005/03/03 22:26:21  dwilliss
 * Added a close method
 *
 * Revision 1.3  2005/03/03 22:18:03  dwilliss
 * *** empty log message ***
 *
 * Revision 1.2  2005/02/24 20:48:39  dwilliss
 * *** empty log message ***
 *
 * Revision 1.1  2005/02/23 21:33:20  linux32build!dwilliss
 * Initial revision
 *
 */

#ifndef INC_MI32_MG2CAD_H
#define INC_MI32_MG2CAD_H

#ifndef INC_RVC_CAD_H
#include <rvc/cad.h>
#endif

#ifndef INC_RVC_STYLE_H
#include <rvc/style.h>
#endif

#ifndef INC_MI32_TRANS2DC_H
#include <mi32/trans2dc.h>
#endif

#ifndef INC_MI32_MGD2_H
#include <mi32/mgd2.h>
#endif
namespace MGD {

class CONTEXT_CAD;

class DEVICE_CAD : public MGD::DEVICE {

	public:

		DEVICE_CAD (
			);

		virtual ~DEVICE_CAD (
			);
	
		ERRVALUE Create (
			RVC::OBJITEM& objitem,
			int height, //!< size in "pixels",
			int width
			);

		ERRVALUE Close (
			);

#ifndef GENERATING_DOXYGEN_OUTPUT
	private:

		RVC::CAD m_CAD;
		RVC::STYLE m_Style;
		COLOR m_CurColor;
		TRANS2D_AFFINE m_Trans;
		int m_NumElements;

		// copy constructor private and unimplemented
		DEVICE_CAD (const DEVICE_CAD&);

		virtual bool v_AvoidFillSpans (
			) const { return true; }
		
		virtual MGD::CAPABILITY v_GetCapabilities (
			) const;

		virtual MGD::CONTEXT* v_CreateContext (
			);
 
		virtual COLORSPACE v_GetColorSpace (
			) const;

		
		virtual ERRVALUE v_FillSpan (
			MGD::CONTEXT* gc,
			INT32 x1,
			INT32 x2,
			INT32 y
			);

		void SetCurColor (
			const COLOR& color
			) { m_CurColor = color; }


		friend class MGD::CONTEXT_CAD;
#endif

	};


class CONTEXT_CAD : public MGD::CONTEXT {
	public:

		~CONTEXT_CAD (
			);


#ifndef GENERATING_DOXYGEN_OUTPUT
	private:
		CONTEXT_CAD (
			MGD::DEVICE_CAD* dev
			);

 		virtual void v_UpdateColor (
			);
 
		virtual void v_UpdateColor2 (
			const COLOR& color
			);

		virtual ERRVALUE v_DrawArc (
			INT32 xcenter,
			INT32 ycenter,
			INT32 xradius,
			INT32 yradius,
			double sangle,					/* Starting angle in RADIANS */
			double eangle,					/* Ending angle in RADIANS */
			double rotangle,				/* Rotation angle in RADIANS */
			MGD::SHAPECLOSE closemode
			);

		virtual ERRVALUE v_FillArc (
			INT32 xcenter,
			INT32 ycenter,
			INT32 xradius,
			INT32 yradius,
			double sangle,					/* Starting angle in RADIANS */
			double eangle,					/* Ending angle in RADIANS */
			double rotangle,				/* Rotation angle in RADIANS */
			MGD::SHAPECLOSE closemode
			);
		
		virtual ERRVALUE v_DrawGlyph (
			MGD::FONT* font,
			double x,
			double y, 
			double baseangle,
			const MGD::STRINGDATA& gidstr,
			int num
			);

		virtual ERRVALUE v_DrawLinesLow (
			const LPOINT2D *lpoint,
			INT32 NumPoints
			); 

		virtual ERRVALUE v_DrawLinesLow (
			const POLYLINE& line
			);
 
		virtual ERRVALUE v_DrawPoint (
			INT32 x,
			INT32 y
			);

		virtual ERRVALUE v_DrawRectangle (
			INT32 x1,
			INT32 y1,
			INT32 x2,
			INT32 y2
			);

		virtual ERRVALUE v_DrawStringStraight (
			const DPOINT2D& pt,	// Base point for the string
			double baseangle,
			const MGD::STRINGDATA& gidstr,
			int sindex,		// The starting glyph index to draw
			int eindex,		// The last glyph index to draw
			double SpaceExtra,
			double GlyphExtra
			);

		virtual ERRVALUE v_FillCircle (
			INT32 xcenter,
			INT32 ycenter,
			INT32 radius
			) { return v_FillArc(xcenter, ycenter, radius, radius, 0.0, 2*PI, 0.0, MGD::SHAPECLOSE_Direct); }

		virtual ERRVALUE v_FillPolygon (
			const LPOINT2D *lpoint,
			INT32 NumPoints,
			MGD::POLYSHAPE shape = MGD::POLYSHAPE_Complex,
			bool DrawBorder = false
			);

		virtual ERRVALUE v_FillPolygons (
			const LPOLYGON *lpolygons,
			INT32 NumPolys,
			bool bDrawBorder
			);


		virtual ERRVALUE v_FillRectangle (
			INT32 x1,
			INT32 y1,
			INT32 x2,
			INT32 y2
			); 

		virtual ERRVALUE v_PutPixel (
			INT32 x,
			INT32 y,
			MGD::PIXEL pixel
			); 
 
		ERRVALUE AddConic (
			INT32 xcenter,
			INT32 ycenter,
			INT32 xradius,
			INT32 yradius,
			double sangle,					/* Starting angle in RADIANS */
			double eangle,					/* Ending angle in RADIANS */
			double rotangle,				/* Rotation angle in RADIANS */
			MGD::SHAPECLOSE closemode,
			bool bFill
			);

		ERRVALUE AddElement (
			RVC::CAD::ELEMENT& elem,		// not const -- will change for styles
			bool bFill = false
			);

		MGD::DEVICE_CAD* m_DevCAD;
		friend class MGD::DEVICE_CAD;
		};

#endif // GENERATING_DOXYGEN_OUTPUT
}



#endif //end define INC_MG2CAD_H
