/**
 * \file unitmenu.h <mi32/unitmenu.h>
 * \brief Header file for unit selection menu functions
 *
 * \if NODOC
 * $Id: unitmenu.h_v 1.18 2003/09/15 13:49:56 fileserver!dwilliss Exp $
 *
 * $Log: unitmenu.h_v $
 * Revision 1.18  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.17  2003/09/05 17:08:19  mju
 * Make most stuff inline as plan to deprecate.
 * Update for UOM system.
 *
 * Revision 1.16  2003/06/03 20:08:14  scowan
 * Added associate with.
 *
 * Revision 1.15  2002/08/14 17:20:07  mju
 * Remove FillInUnit...ComboBox as unit one has method equivalent in combobox and type one is a trivial loop.
 *
 * Revision 1.14  2001/12/13 16:59:14  mju
 * MGUI class change.
 *
 * Revision 1.13  2001/10/22 21:05:25  dwilliss
 * Fixed error in name on ifdef
 *
 * Revision 1.12  2001/10/22 20:53:42  dwilliss
 * Added prototypes for MFC version
 *
 * Revision 1.11  2000/09/01 20:35:43  msmith
 * Genitor documentation
 *
 * Revision 1.10  2000/08/31 21:43:00  mju
 * INclude xdefns.h so compiles in isolation.
 *
 * Revision 1.9  1999/04/08 19:18:02  mju
 * Add inclusion guards.
 *
 * Revision 1.8  1999/01/05  19:12:56  scowan
 * Swapped multiple keywords
 *
 * Revision 1.7  1998/10/13 17:22:48  mju
 * Add UNITMENU_UseSymbol and UNITMENU_OneOfMany.
 *
 * Revision 1.6  1998/03/18  16:34:36  mju
 * Update callback functions to take parameters in prototypes.
 *
 * Revision 1.5  1998/03/05  21:36:07  mju
 * Rework for i18n.
 * \endif
**/

#ifndef	INC_MI32_UNITMENU_H
#define	INC_MI32_UNITMENU_H

#ifndef	INC_MI32_UNITCONV_H
#include <mi32/unitconv.h>
#endif

#ifdef X_NATIVE
#ifndef	INC_MI32_XDEFNS_H
#include <mi32/xdefns.h>
#endif
#endif


#define	UNITMENU_UseSymbol		0x0001
#define	UNITMENU_OneOfMany		0x0002
#define	UNITMENU_NameSingular	0x0004
#define	UNITMENU_AddNone			0x1000

//!:Associate with "Unit Conversions"
//!\addtogroup UnitConv Unit Conversion
//!@{

#if defined(__cplusplus)
extern "C" {
#endif

#ifdef X_NATIVE
//! Create MENUITEM array for all defined units of a specified type.
//!
//! @return MENUITEM array containing unit names
//! Flags
//! 	\li\b UNITMENU_UseSymbol					Use symbol instead of name for unit
//! 	\li\b UNITMENU_OneOfMany					Set MxMENU_ONE_OF_MANY in each MENUITEM
//!		\li\b UNITMENU_NameSingular				Use singular (eg meter, foot) for unit name instead of plural
//! 	\li\b UNITMENU_AddNone						Add "None" item to menu array
//! Returns an array of MENUITEMs for units of the specified type.  If flags is MxONE_OF_MANY, the
//! array will contain radio buttons.
//! The callback function will get as its 2nd parameter a number.  This number (call it "unit")
//! represents the unit type AND the unit number.  unit/256 is the unit type, unit%256 is
//! the unit number.
MENUITEM * MxCreateUnitMenuItems (
	int type,									//!< Unit type
	int dft,										//!< Default unit number
	XtCallbackProc cbfunc,					//!< Callback function
	UINT32 flags = 0							//!< Flags (UNITMENU_...)
	);

//! Create OptionMenu for specifying units (feet, meters, etc...).
//! @return OptionMenu widget, unmanaged if no arglist provided, otherwise managed.
//! Returns an OptionMenu widget for selecting units of a given type.
//! The callback function will get as its 2nd parameter a number.  This number (call it "unit")
//! represents the unit type AND the unit number.  unit/256 is the unit type, unit%256 is the unit number.
//! Flags
//!		\li\b UNITMENU_UseSymbol					Use symbol instead of name for unit
//! 	\li\b UNITMENU_OneOfMany					Set MxMENU_ONE_OF_MANY in each MENUITEM
//!		\li\b UNITMENU_NameSingular				Use singular (eg meter, foot) for unit name instead of plural
//! 	\li\b UNITMENU_AddNone						Add "None" item to menu array
Widget MxCreateUnitOptionMenu (
	Widget pwidget,							//!< Parent widget to contain OptionMenu
	const char *label,						//!< Label of OptionMenu
	int type,									//!< Unit type
	int dft,										//!< Default unit number
	XtCallbackProc cbfunc,					//!< Callback function
	void *udata,								//!< Value to use for menu item XmNuserData resource
	UINT32 flags = 0,							//!< Flags (UNITMENU_...)
	const Arg *arglist = 0,					//!< Arguments for widget attachment
	int argcount = 0							//!< Number of arguments
	);

//! Create OptionMenu for specifying units (feet, meters, etc...).
//! @see MxCreateUnitOptionMenu.
inline Widget MxCreateUnitOptionMenuExt (
	Widget pwidget,							//!< Parent widget to contain OptionMenu
	const char *label,						//!< Label of OptionMenu
	int type,									//!< Unit type
	int dft,										//!< Default unit number
	XtCallbackProc cbfunc,					//!< Callback function
	void *udata,								//!< Value to use for menu item XmNuserData resource
	UINT32 flags,								//!< Flags (UNITMENU_...)
	const Arg *arglist,						//!< Arguments for widget attachment
	int argcount								//!< Number of arguments
	) {
	return (MxCreateUnitOptionMenu(pwidget,label,type,dft,cbfunc,udata,flags,arglist,argcount));
	}

//! Create OptionMenu for specifying units (feet, meters, etc...).
//! @see MxCreateUnitOptionMenu.
inline Widget MxCreateUnitOptionMenuF (
	Widget pwidget,							//!< Parent widget
	const char *label,						//!< Label of OptionMenu
	int type,									//!< Unit type
	int dft,										//!< Default unit number
	XtCallbackProc cbfunc,					//!< Callback function
	void *udata,								//!< Value to use for menu item XmNuserData resource
	UINT32 flags								//!< Flags (UNITMENU_...)
	) {
	return (MxCreateUnitOptionMenu(pwidget,label,type,dft,cbfunc,udata,flags,NULL,0));
	}
#endif

#if defined(__cplusplus)
}
#endif

//!@}

#endif	//!<  INC_MI32_UNITMENU_H 
