/**
 * \file rvc/cmap.h 
 * \brief interface for the RVC::COLORMAP class.
 *
 * \if NODOC
 * $Id: cmap.h_v 1.18 2007/09/06 17:18:35 scowan Exp $
 *
 * $Log: cmap.h_v $
 * Revision 1.18  2007/09/06 17:18:35  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.17  2007/01/19 16:10:14  mju
 * Remove legacy method palette.
 *
 * Revision 1.16  2007/01/18 14:43:51  mju
 * Use color_array.
 *
 * Revision 1.15  2006/10/13 19:24:11  mju
 * Use getobjfilter hdr.
 *
 * Revision 1.14  2005/12/19 20:37:07  scowan
 * Added isselectable class.
 *
 * Revision 1.13  2005/12/09 17:29:16  scowan
 * Updated to use credentials.
 *
 * Revision 1.12  2005/07/19 20:51:24  scowan
 * Fixed virtual method issue.
 *
 * Revision 1.11  2005/01/19 23:21:26  scowan
 * Support of descriptor class.
 *
 * Revision 1.10  2004/03/11 17:23:30  scowan
 * Added generate std color method.
 *
 * Revision 1.9  2003/10/07 17:35:47  scowan
 * Object virtual method changes.
 *
 * Revision 1.1  2000/09/29 15:37:49  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_CMAP_H
#define INC_RVC_CMAP_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

#ifndef INC_MI32_COLORARRAY_H
	#include <mi32/colorarray.h>
#endif

#ifndef INC_RVC_GETOBJFILTER_H
	#include <rvc/getobjfilter.h>
#endif

namespace RVC {

//!	\class RVC::COLORMAP rvc/cmap.h
//!	\brief Manages RVC::OBJTYPE_ColorPalette objects.  
//!
//!	RVC Object Types that this class supports: OBJTYPE_ColorPalette.
//!
//!	Default RVC Object Type for this class: OBJTYPE_ColorPalette.
//!
//!	Valid parents for this class:
//!		- OBJTYPE_File
//!		- OBJTYPE_Folder
//!		- OBJTYPE_Raster
//!
//!	Read(), Make() and Write() methods inherited from RVCSIMPLE class.  View RVCSIMPLE docs for inherited methods
class COLORMAP : public RVC::SIMPLE {
	public:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class GETOBJFILTER_ISSELECTABLE;
		#endif // GENERATING_DOXYGEN_OUTPUT

		COLORMAP (
			);

		COLORMAP (
			const RVC::COLORMAP& rhs
			);

		virtual ~COLORMAP (
			);

		RVC::COLORMAP& operator= (
			const RVC::COLORMAP& rhs
			);

		//! Generate the default 80 standard color colormap
		//! Can be used to Make() an object with the 80 std colors
		ERRVALUE GenerateStandardColors (
			);

		//! Obtain color map from the instance, not useful until Read() method is performed
		//! @return Constant Reference to colormap array.
		const COLOR_ARRAY& GetColorMap (
			) const;

		//! Set the internal colormap array storage for later writing
		ERRVALUE SetColorMap (
			const COLOR_ARRAY& ColorMap
			);

	protected:

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;

		//! Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS& ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);

		COLOR_ARRAY m_ColorMap;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};


class COLORMAP::GETOBJFILTER_ISSELECTABLE : public GETOBJFILTER {
	public:
		GETOBJFILTER_ISSELECTABLE ();
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool v_HasSelectableSubObjects (const RVC::OBJITEM& objitem);
		virtual bool v_IsSelectable (const RVC::OBJITEM& ObjItem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		//! End of RVC namespace

#endif			//!< INC_RVCCOLORMAP_H
