/**
 * \file rvc/credentials.h
 * \brief Definitions header file for user credentials
 *
 * \if NODOC
 * $Id: credentials.h_v 1.4 2011/09/02 12:53:01 mju Exp $
 *
 * $Log: credentials.h_v $
 * Revision 1.4  2011/09/02 12:53:01  mju
 * Dox hdr.
 *
 * Revision 1.3  2007/01/12 22:24:08  scowan
 * Added comparison operators.
 *
 * Revision 1.2  2006/02/10 22:42:52  scowan
 * Place openfile and inodeinst into rvc namespace.
 *
 * Revision 1.1  2005/12/09 17:29:19  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_CREDENTIALS_H
#define	INC_RVC_CREDENTIALS_H

#ifndef INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class MISTRING;
namespace RVC {
	class OPENFILE;
	}
#endif // GENERATING_DOXYGEN_OUTPUT

namespace RVC {

class RVCSYSLIBCLASSEXPORT CREDENTIALS {
	public:
		//! Default constructor, credentials set to "Everyone"
		CREDENTIALS (
			);
			
		//! Constructor that takes a user name and password to set credentials
		CREDENTIALS (
			const MISTRING& UserName,
			MISTRING& Password					//!< String cleared after credentials are set
			);

		//! Copy constructor
		CREDENTIALS (
			const CREDENTIALS& rhs
			);
		
		//! Destructor
		~CREDENTIALS (
			);
			
		//! Assignment operator
		CREDENTIALS& operator= (
			const CREDENTIALS& rhs
			);
		
		//! Comparison operator (Equals)
		bool operator== (
			const CREDENTIALS& rhs
			) const;
		
		//! Comparison operator (Not Equals)
		bool operator!= (
			const CREDENTIALS& rhs
			) const { return (!(*this == rhs)); }
		
		//! Set credentials via user name and password
		void SetCredentials (
			const MISTRING& UserName,
			MISTRING& Password					//!< String cleared after credentials are set
			);

	private:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV* m_Priv;
		friend class OPENFILE;
		#endif //!< GENERATING_DOXYGEN_OUTPUT
	};

}		// End of RVC namespace

#undef RVCSYSLIBEXPORT
#undef RVCSYSLIBCLASSEXPORT

#endif		// INC_RVC_CREDENTIALS_H
