/**
 * \file rvc/descriptor.h
 * \brief RVC::DESCRIPTOR class definitions
 *
 * $Id: descriptor.h_v 1.28 2011/10/10 20:46:04 mju Exp $
 *
 * $Log: descriptor.h_v $
 * Revision 1.28  2011/10/10 20:46:04  mju
 * Remove unneeded hdrs.
 *
 * Revision 1.27  2011/05/27 20:50:22  mju
 * Add constructor with name/desc.
 *
 * Revision 1.26  2007/09/25 21:33:39  scowan
 * Error fixed in inline operator<.
 *
 * Revision 1.25  2007/09/25 21:24:22  scowan
 * Added isLessThan and inline operator< methods.
 *
 * Revision 1.24  2006/04/18 15:18:37  mju
 * Add setName variant using delegate to ensure generated short name unique.
 *
 * Revision 1.23  2006/01/25 14:47:05  mju
 * Don't pragma dll warning as done in stddefns.
 *
 * Revision 1.22  2005/11/10 16:38:33  scowan
 * Uninlined the dtor because the compiler will not inline it.
 *
 * Revision 1.21  2005/07/26 21:59:10  scowan
 * *** empty log message ***
 *
 * Revision 1.20  2005/07/26 21:57:07  scowan
 * Added methods.
 *
 * Revision 1.19  2005/07/25 20:09:01  scowan
 * Do not include rvc defns.
 *
 * Revision 1.18  2005/07/21 14:09:31  scowan
 * Changed to support fixedstr delegate chagnes.
 *
 * Revision 1.17  2005/07/20 20:26:13  scowan
 * Made private method public.
 *
 * Revision 1.16  2005/07/18 22:53:45  scowan
 * Added another friend.
 *
 * Revision 1.15  2005/07/08 14:51:16  scowan
 * Added method to set only the short name.
 *
 * Revision 1.14  2005/06/22 19:12:00  scowan
 * Made more changes.
 *
 * Revision 1.13  2005/06/22 17:31:32  scowan
 * Moved getSource method to dll internal.
 *
 * Revision 1.12  2005/06/20 19:11:02  scowan
 * Adjusted include files.
 *
 * Revision 1.11  2005/03/08 15:33:37  mju
 * Add op= from textid.
 *
 * Revision 1.10  2005/02/22 23:24:34  scowan
 * Nothing.
 *
 * Revision 1.9  2005/01/21 22:16:40  scowan
 * Moved set source to internal.
 *
 * Revision 1.8  2005/01/19 16:04:50  mju
 * Add ctor using objectname so existing code compiles.
 *
 * Revision 1.7  2005/01/18 23:31:48  scowan
 * Added a friend.
 *
 * Revision 1.6  2005/01/18 21:22:00  scowan
 * Fixed warnings.
 *
 * Revision 1.5  2005/01/13 20:52:12  scowan
 * Update docs.
 *
 * Revision 1.4  2005/01/13 20:48:41  scowan
 * Added assignment from mi string.
 *
 * Revision 1.3  2005/01/13 17:38:11  scowan
 * Added Mike's changes.
 *
 * Revision 1.2  2005/01/13 17:11:03  scowan
 * Added docs and moved to dft ctor.
 *
 * Revision 1.1  2005/01/12 22:47:31  scowan
 * Initial revision
 *
**/

#ifndef	INC_RVC_DESCRIPTOR_H
#define	INC_RVC_DESCRIPTOR_H

#ifndef INC_MI32_FIXEDSTR_H
	#include <mi32/fixedstr.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif

namespace RVC {

typedef FIXEDSTRING<16>	OBJECTNAME;

//! DESCRIPTOR class for specifying name, description, and source for an object
class RVCSYSLIBCLASSEXPORT DESCRIPTOR {
	public:

		//! Default constructor
		//! Presets the 'source' field from MsysGetAppName().
		DESCRIPTOR ();

		//! Constructor taking a name as a MISTRING.
		//! Will fill in the name field and short name fi with validating the name (does not guarentee uniqueness).
		//! Will set the description field if the resulting short name is different than 'name'
		//! Presets the 'source' field from MsysGetAppName().
		DESCRIPTOR (
			const MISTRING& name
			);

		//! Constructor taking a name as a TEXTID.
		//! Will fill in the name field and short name field with validating the name (does not guarentee uniqueness).
		//! Will set the description field if the resulting short name is different than 'name'
		//! Presets the 'source' field from MsysGetAppName().
		DESCRIPTOR (
			TEXTID TextID
			);

		//! Construct with both name and description.
		DESCRIPTOR (
			const MISTRING& name,
			const MISTRING& description
			);

		//! Constructor taking a name as a RVC::OBJECTNAME.
		DEPRECATED DESCRIPTOR (
			const RVC::OBJECTNAME& name
			);

		//! Copy constructor
		DESCRIPTOR (
			const DESCRIPTOR& rhs
			) :
			m_Name(rhs.m_Name),
			m_ShortName(rhs.m_ShortName),
			m_Description(rhs.m_Description),
			m_Source(rhs.m_Source)
			{}

		//! Destructor
		~DESCRIPTOR ();

		//! Assignment operator
		DESCRIPTOR& operator= (
			const DESCRIPTOR& rhs
			) {
			if (this != &rhs) {
				m_Name = rhs.m_Name;
				m_ShortName = rhs.m_ShortName;
				m_Description = rhs.m_Description;
				m_Source = rhs.m_Source;
				}
			return (*this);
			}
			
		//! Assignment from MISTRING, will clear out previous settings.
		//! Will fill in the name field and short name field with validating the name (does not guarentee uniqueness).
		//! Will set the description field if the resulting short name is different than 'name'
		//! Presets the 'source' field from MsysGetAppName().
		DESCRIPTOR& operator= (
			const MISTRING& name
			);
			
		//! Assignment from TEXTID, will clear out previous settings.
		//! Will fill in the name field and short name field with validating the name (does not guarentee uniqueness).
		//! Will set the description field if the resulting short name is different than 'name'
		//! Presets the 'source' field from MsysGetAppName().
		DESCRIPTOR& operator= (
			TEXTID textid
			);
			
		//! Clear the instance of all strings
		void Clear (
			) {
			m_Name.Clear();
			m_ShortName.Clear();
			m_Description.Clear();
			m_Source.Clear();
			return;
			}

		//! Get the description of the object
		//! @return Constant reference to the description
		const MISTRING& GetDescription (
			) const { return (m_Description); }
			
		//! Get the name of the object
		//! @return Constant reference to the name
		const MISTRING& GetFullName (
			) const { return (m_Name); }

		//! Get the short name of the object
		//! @return Constant reference to the short name
		const OBJECTNAME& GetShortName (
			) const { return (m_ShortName); }

		//! Get the source of the object
		//! @return Constant reference to the source
		const OBJECTNAME& GetSource (
			) const;

		//! Is the descriptor empty?
		bool IsEmpty (
			) const { return (m_Name.IsEmpty() && m_Description.IsEmpty()); }
			
		//! Are the descriptor's equal?
		bool IsEqual (
			const DESCRIPTOR& rhs
			) const;

		//! Are the descriptor's equal?
		bool IsLessThan (
			const DESCRIPTOR& rhs
			) const { return (m_Name < rhs.m_Name || (m_Name == rhs.m_Name && m_Description < rhs.m_Description)); }

		//! Set the object description
		void SetDescription (
			const MISTRING& Description			//!< The MISTRING containing the long description
			) { m_Description = Description; }
			
		//! Set the object name.
		//! This will set the short name value as well.
		void SetName (
			const MISTRING& Name						//!< The MISTRING containing the long name
			);

		//! Set the object name with check for uniquely generated short name.
		//! This will set the short name value as well.
		//! Delegate will only be called if the generated short name has changed and is not identical to the full name.
		void SetName (
			const MISTRING& Name,					//!< The MISTRING containing the long name
			FastDelegate<int(const RVC::OBJECTNAME&)> DelegateTestUniqueShortName
			);

		//! Set the short name (object ID tag)
		void SetShortName (
			const OBJECTNAME& name
			);
			
		//! Set the object source
		void SetSource (
			const OBJECTNAME& Source				//!< The OBJECTNAME containing the source string
			) { m_Source = Source; }

		//! Set the object source
		//! Pass NULL for the 'Source' parameter to set to MsysGetAppName().
		void SetSource (
			const char* Source						//!< The ASCII string containing the source string
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_Name;
		MISTRING m_Description;
		OBJECTNAME m_ShortName;
		mutable OBJECTNAME m_Source;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
}		// End of RVC namespace

#undef RVCSYSLIBCLASSEXPORT

inline bool operator== (
	const RVC::DESCRIPTOR& lhs,
	const RVC::DESCRIPTOR& rhs
	) { return (lhs.IsEqual(rhs)); }

inline bool operator!= (
	const RVC::DESCRIPTOR& lhs,
	const RVC::DESCRIPTOR& rhs
	) { return (!lhs.IsEqual(rhs)); }

//! Used for STL containers that sort their items
inline bool operator< (
	const RVC::DESCRIPTOR& lhs,
	const RVC::DESCRIPTOR& rhs
	) { return (lhs.IsLessThan(rhs)); }

#endif		// INC_RVC_DESCRIPTOR_H
