/**
* \file rvc/opvectorgridgeneration.h
*
* \if NODOC
* $Id: opvectorgridgeneration.h_v 1.12 2012/10/22 20:20:52 mju Exp $
*
* $Log: opvectorgridgeneration.h_v $
* Revision 1.12  2012/10/22 20:20:52  mju
* Remove unneeded parm fields and use accessor methods for all others.
*
* Revision 1.11  2011/12/22 19:06:53  mju
* Remove redundant hdrs.
*
* Revision 1.10  2011/12/08 15:34:45  mju
* Use unitconv hdr.
*
* Revision 1.9  2011/06/15 21:44:55  mju
* Clean up and hide private stuff from docs.
*
* Revision 1.8  2010/04/26 14:28:02  mju
* Use unsignedset.
* Make internal structures private.
*
* Revision 1.7  2007/08/29 17:15:50  bcolombe
* Made some methods public  to be used for computing number of lanes
*
* Revision 1.1  2007/04/09 17:16:31  drb
* Initial revision
* \endif
**/

#ifndef	INC_RVC_OPVECTORGRIDGENERATION_H
#define	INC_RVC_OPVECTORGRIDGENERATION_H

#ifndef INC_RVC_VECTTOOL_H
	#include <rvc/vecttool.h>
#endif

#ifndef	INC_MI32_UNITCONV_H
	#include <mi32/unitconv.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class UNSIGNEDSET;
class QUADTREE;
struct GRIDENUMDEF2;
#endif


namespace RVC {
namespace OP {

class VECTOR_GRID_GENERATION {
	public:

		enum MODE {
			MODE_None = 0,
			MODE_Triangle,
			MODE_Square,
			MODE_Rectangle,
			MODE_Hexagon,
			MODE_Parallel,
			MODE_RandomRectangle,
			MODE_Count
			};

		enum ENUMERATION {
			ENUMERATION_SymbolicColumn = 0,
			ENUMERATION_SymbolicRow,
			ENUMERATION_SymbolicColumnRow,
			ENUMERATION_NumericRow,
			ENUMERATION_NumericColumn,
			ENUMERATION_NumericRowColumn,
			ENUMERATION_NumericSequential,
			ENUMERATION_NumericSerpentine,
			ENUMERATION_Count
			};

		class GRID_PARMS {
			public:

				GRID_PARMS ();
				GRID_PARMS (VECTOR_GRID_GENERATION::MODE mode);
				~GRID_PARMS () { }

				void CopyOrientParmsFrom (
					const VECTOR_GRID_GENERATION::GRID_PARMS& OrientParms
					);

				bool GetBuildTables () const
					{ return (m_BuildTables); }

				VECTOR_GRID_GENERATION::MODE GetMode () const
					{ return (m_mode); }

				double GetOrientAngle () const
					{ return (m_OrientAngle); }

				const DPOINT2D& GetOrientStart () const
					{ return (m_OrientStart); }

				void SetBuildTables (
					bool build 
					) { m_BuildTables = build; }

				void SetOrientAngle (
					double angle 
					) { m_OrientAngle = angle; }

				void SetOrientStart (
					const DPOINT2D& start 
					) { m_OrientStart = start; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				VECTOR_GRID_GENERATION::MODE m_mode;
				DPOINT2D m_OrientStart;
				double m_OrientAngle;
				bool m_BuildTables;
				#endif
			};

		class TRIANGLE_PARMS : public GRID_PARMS {
			public:

				TRIANGLE_PARMS ();

				double GetLengthOfSide () const
					{ return (m_LengthOfSide); }

				void SetLengthOfSide (double LengthOfSide)
					{ m_LengthOfSide = LengthOfSide; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				double m_LengthOfSide;
				#endif
			};

		class RECTANGLE_PARMS : public GRID_PARMS {
			public:

				RECTANGLE_PARMS ();

				const DPOINT2D& GetSize () const
					{ return (m_Size); }		

				void SetSize (const DPOINT2D& Size)
					{ m_Size = Size; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				DPOINT2D m_Size;
				#endif
			};

		class HEXAGON_PARMS : public GRID_PARMS {
			public:

				HEXAGON_PARMS ();

				double GetLengthOfSide () const
					{ return (m_LengthOfSide); }

				void SetLengthOfSide (double LengthOfSide)
					{ m_LengthOfSide = LengthOfSide; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				double m_LengthOfSide;
				#endif
			};

		class RANDOMRECTANGLE_PARMS : public GRID_PARMS {
			public:

				RANDOMRECTANGLE_PARMS ();

				const DPOINT2D& GetMaxSize () const
					{ return (m_MaxSize); }

				const DPOINT2D& GetMinSize () const
					{ return (m_MinSize); }

				void SetMaxSize (const DPOINT2D& MaxSize)
					{ m_MaxSize = MaxSize; }

				void SetMinSize (const DPOINT2D& MinSize)
					{ m_MinSize = MinSize; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				DPOINT2D m_MinSize;
				DPOINT2D m_MaxSize;
				#endif
			};

		class PARALLEL_PARMS : public GRID_PARMS {
			public:
				PARALLEL_PARMS ();

				bool GetIgnoreOrigin (
					) { return (m_ignoreOrigin); }

				double GetWidth () const
					{ return (m_Width); }
				
				void SetIgnoreOrigin (
					bool ignoreOrigin
					) { m_ignoreOrigin = ignoreOrigin; }
				
				void SetWidth (double Width)
					{ m_Width = Width; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				double m_Width;
				bool m_ignoreOrigin;
				#endif
			};

		//! Default constructor.
		VECTOR_GRID_GENERATION ();

		//! Destructor.
		~VECTOR_GRID_GENERATION ();

		ERRVALUE ComputeLRWidth (double *LWidth, double *RWidth, GRID_PARMS* GridParms);

		ERRVALUE MakeQTreeOfBoundary (RVC::VECTOR& VectObj, const UNSIGNEDSET *pSelectedPolygonSet = 0);

		ERRVALUE Process (
			RVC::VECTOR& VectObj, 
			RVC::OBJITEM& GridObj, 
			GRID_PARMS *GridParms, 
			const UNSIGNEDSET *pSelectedPolygonSet = 0
			); 

		ERRVALUE SetGridScale ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		struct GRIDSHAPE {
			DPOINT2D origin;
			DPOINT2D vect1;
			INT32 num1;
			INT32 enum1;
			DPOINT2D vect2;
			INT32 num2;
			INT32 enum2;
			INT32 numSegm;
			INT32 ia[5];
			INT32 ja[5];
			int s[5];
			int asign;
			DPOINT2D va;
			INT32 im;
			INT32 jm;
			INT32 numPoints;
			DOUBLE_ARRAY<DPOINT2D> GPoints;
			DOUBLE_ARRAY<DPOINT2D> APoints;
			double minv1;
			double maxv1;
			double minv2;
			double maxv2;
			DOUBLE_ARRAY<double> randomv1;
			DOUBLE_ARRAY<double> randomv2;
			DRECT2D rect;
			double tol;
			QUADTREE* qth;
			INT32 numseg;
			INT32 maxseg;
			UINT8 *flagseg;

			ERRVALUE GetSegment (INT32 num, DPOINT2D *vect);
			};

		class EXTSTR {
			public:

				EXTSTR ();
				~EXTSTR ();

				ERRVALUE ComputeConvexIn ();
				ERRVALUE GetRadiusIn (const DPOINT2D& Point, double theta, double beta, double *Radius);
				ERRVALUE InsertTo (const DPOINT2D& Point);

			private:
				int m_InsFlag;
				DPOINT2D m_GPoint;
				DOUBLE_ARRAY<DPOINT2D> m_Points;
			};

		struct TOPOLINE {
			INT32 num;
			INT32 left;
			INT32 right;
			UINT8 flag;
			};

		RVC::VECTOR m_BorderObj;
		RVC::VECTOR m_GridObj;
		QUADTREE *m_qth;
		UINT8 *m_flagseg;
		GRID_PARMS* m_GridParms;
		double m_xscale;
		double m_yscale;
		GRIDSHAPE m_shape;
		INT32 m_numlines;
		SIMPLE_ARRAY<TOPOLINE> m_topoline;
		INT32 m_numpolys;
		INT32 m_numseg;
		INT32 m_numselseg;
		EXTSTR *m_pExtStr;
		bool m_gefields[ENUMERATION_Count];

		ERRVALUE BuildGridCellDataTable ();
		ERRVALUE CopyQTreeOfBoundary ();
		ERRVALUE DivideSegment (DPOINT2D *Point, DRECT2D  *ext, DOUBLE_ARRAY<DPOINT2D>&Points, INT32 *count);
		void FreeGridShapePoints ();
		void FreeGridShapeQTree ();
		void FreeQtreeOfBoundary ();
		ERRVALUE GenerateExtStr (const UNSIGNEDSET *pSelectedPolygonSet);
		ERRVALUE GenerateIntersectionPoints (DPOINT2D *Point, DPOINT2D **Points);
		ERRVALUE GenerateRowColumnRecord (DPOINT2D Point, INT32 *row, INT32 *col, GRIDENUMDEF2 *genum);
		void GenerateRowRecord (INT32 r, GRIDENUMDEF2 *genum);
		UINT8 GetFlagSegIndexed (INT32 index) const
			{ return m_flagseg[index]; }
		void GetRowColumnCoor (DPOINT2D Point, INT32 *row, double *col);
		QUADTREE* GetQuadTreeHandle () const
			{ return m_qth; }
		double GetXScale () const
			{ return m_xscale; }
		double GetYScale () const
			{ return m_yscale; }
		ERRVALUE InsideArea (double x, double y);
		ERRVALUE MakeGrid ();
		ERRVALUE MakeGridPoints ();
		ERRVALUE MakeGridShape ();
		ERRVALUE MakeGridVector (RVC::OBJITEM& GridObjItem);
		void MakeScaledVector (DPOINT2D *vect);
		ERRVALUE Segmentation (DPOINT2D *Point, DRECT2D  *rect, INT32 *count);
		ERRVALUE SnapGridPointsToBoundary ();
		ERRVALUE WriteBoundary (INT32 *count);
		ERRVALUE WriteSegment (INT32 count, DPOINT2D *vect);

		#endif	// GENERATING_DOXYGEN_OUTPUT
  	};

} // End namespace OP
} // End namespace RVC


#endif	// INC_RVC_OPVECTORGRIDGENERATION_H
