/**
 * \file rvc/uidlgdbfieldrevalue.h
 * \brief RVC::UI::DLG_DB_FIELD_REVALUE definitions
 *
 * \if NODOC
 * $Id: uidlgdbfieldrevalue.h_v 1.5 2010/09/13 14:18:01 mju Exp $
 *
 * $Log: uidlgdbfieldrevalue.h_v $
 * Revision 1.5  2010/09/13 14:18:01  mju
 * Add canApply.
 *
 * Revision 1.4  2010/09/08 14:23:09  mju
 * Add dlgstyle_AllowRedrawOnApply.
 *
 * Revision 1.3  2009/08/28 21:59:48  mju
 * Add statusbar.
 *
 * Revision 1.2  2009/08/21 16:18:10  mju
 * *** empty log message ***
 *
 * Revision 1.1  2009/08/21 13:35:02  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIDLGDBFIELDREVALUE_H
#define	INC_RVC_UIDLGDBFIELDREVALUE_H

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_STATUSBAR_H
#include <mgui/statusbar.h>
#endif

#ifndef	INC_RVC_UIFORMDBFIELDPICKLIST_H
#include <rvc/uiformdbfieldpicklist.h>
#endif

namespace RVC {
namespace UI {
//=====================================================================================================================

//! Base dialog to allow user to revalue fields.
//! Controls are provided to let user pick existing values along with a new value.
//! Override the v_OnApply to handle the Apply/OK buttons.
class DLG_DB_FIELD_REVALUE : public MGUI::DLGSHELL {
	public:

		enum DLGSTYLE {
			DLGSTYLE_Default =					0,
			DLGSTYLE_AllowRedrawOnApply =		0x01,			//!< Include toggle for "Redraw after apply changes"
			};

		//! Constructor.
		explicit DLG_DB_FIELD_REVALUE (
			const MISTRING& title,
			DLGSTYLE dlgstyle = DLGSTYLE_Default
			);

		//! Destructor.
		virtual ~DLG_DB_FIELD_REVALUE ();

		//! Apply change to record if matches user selection.
		//! @return true if changed, false if not.
		bool ApplyToRecord (
			RVC::DBTABLE::RECORD& record			//!< Record to place value into
			) const;

		//! Apply change to specified record number if matches user selection.
		//! @return TRUE if changed, FALSE if not, < 0 if error.
		int ApplyToRecordNum (
			INT32 RecordNum
			);

		//! Determine if can apply reclassification.
		bool CanApply () const
			{ return (m_FormChangeFrom.HasSelection() && m_FormChangeTo.HasSelection()); }

		//! Get field number being revalued.
		INT32 GetFieldNumFor (
			) const { return (m_FieldNumFor); }

		//! Get whether to redraw after apply.
		bool GetRedrawOnApply () const
			{ return ((m_dlgstyle & DLGSTYLE_AllowRedrawOnApply) && m_Settings.GetRedrawOnApply()); };

		//! Get table revaluing field for.
		const RVC::DBTABLE& GetTableFor (
			) const { return (m_ObjTableFor); }

		//! Get statusbar control.
		MGUI::CTRL_STATUSBAR& GetStatusBar (
			) { return (m_StatusBar); }

		//! Save current settings if changed.
		void SaveSettings ();

		//! Setup with table and field to revalue.
		//! Field must related to a primary key from which values are selected.
		//! Must be done only once and before dialog is created.
		ERRVALUE Setup (
			const RVC::DBTABLE& ObjTable,			//!< Table revalue field for
			INT32 FieldNum,							//!< Field to revalue
			INT32 RelFieldDisplay = -1,			//!< Field number in related picklist table to display values from
			INT32 RelFieldSortBy = -1				//!< Field number in related picklist table to sort by
			);

	protected:

		//! Called when dialog closed - derived class should call base method to save settings.
		virtual void v_OnClose ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef DLG_DB_FIELD_REVALUE THISCLASS;

		class SETTINGS {
			public:
				static const SERIALIZERITEM* SerialGetItemDef ();

				SETTINGS (
					);
				~SETTINGS ();

				const RVC::UI::FORM_DB_FIELD_PICKLIST::SETTINGS& GetChangeFrom (
					) const { return (m_SettingsChangeFrom); }
				const RVC::UI::FORM_DB_FIELD_PICKLIST::SETTINGS& GetChangeTo (
					) const { return (m_SettingsChangeTo); }
				const LPOINT2D& GetPosition (
					) const { return (m_Position); }
				bool GetRedrawOnApply () const
					{ return (m_RedrawOnApply); }
				bool HasPosition (
					) const { return (m_Position.x >= 0 && m_Position.y >= 0); }
				bool IsChanged (
					) const { return (m_IsChanged); }
				void ResetChanged (
					) { m_IsChanged = false; }
				void SetChangeFrom (const RVC::UI::FORM_DB_FIELD_PICKLIST::SETTINGS& ChangeFrom);
				void SetChangeTo (const RVC::UI::FORM_DB_FIELD_PICKLIST::SETTINGS& ChangeTo);
				void SetFieldNumDisplay (INT32 FieldNum);
				void SetFieldNumSortBy (INT32 FieldNum);
				void SetPosition (
					INT32 x,
					INT32 y
					) { if (x != m_Position.x || y != m_Position.y) { m_IsChanged = true; m_Position.x = x; m_Position.y = y; } }
				void SetRedrawOnApply (
					bool RedrawOnApply
					) { if (RedrawOnApply != m_RedrawOnApply) { m_IsChanged = true; m_RedrawOnApply = RedrawOnApply; } }
			private:
				typedef SETTINGS THISCLASS;
				bool m_IsChanged;
				bool m_RedrawOnApply;
				LPOINT2D m_Position;
				RVC::UI::FORM_DB_FIELD_PICKLIST::SETTINGS m_SettingsChangeFrom;
				RVC::UI::FORM_DB_FIELD_PICKLIST::SETTINGS m_SettingsChangeTo;
			};

		DLGSTYLE m_dlgstyle;
		MISTRING m_ObjNameSettings;
		SETTINGS m_Settings;
		RVC::DBTABLE m_ObjTableFor;
		RVC::DBTABLE::RECORD m_RecordFor;
		INT32 m_FieldNumFor;
		RVC::DBTABLE m_ObjTablePickFrom;
		INT32 m_FieldNumPickFrom;
		bool m_AllowNone;
		RVC::UI::FORM_DB_FIELD_PICKLIST m_FormChangeFrom;
		RVC::UI::FORM_DB_FIELD_PICKLIST m_FormChangeTo;
		MGUI::CTRL_TOGGLEBUTTON m_TglRedrawOnApply;
		MGUI::CTRL_STATUSBAR m_StatusBar;

		void EnableButtons ();
		void LoadSettings ();
		void OnTglRedrawOnApply ();

		// SHELL/DLGSHELL overrides.
		virtual ERRVALUE v_CreateContent ();

		#endif
	};

//=====================================================================================================================
}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIDLGDBFIELDREVALUE_H

