/**
 * \file rvc/uiformselectchild.h
 * \brief RVC::UI::FORM_SELECT_CHILD class definitions
 *
 * \if NODOC
 * $Id: uiformselectchild.h_v 1.9 2011/09/12 16:38:35 mju Exp $
 *
 * $Log: uiformselectchild.h_v $
 * Revision 1.9  2011/09/12 16:38:35  mju
 * Add hasSelectedChild.
 *
 * Revision 1.8  2011/09/09 17:59:10  mju
 * Add addChildren using objitem.
 *
 * Revision 1.7  2011/09/09 17:46:56  mju
 * Add deleteChildren.
 *
 * Revision 1.6  2011/09/09 17:20:41  mju
 * Redesign so allow use of combobox::addItem.
 *
 * Revision 1.5  2008/09/02 21:42:47  mju
 * Fix doxygen warning.
 *
 * Revision 1.1  2007/03/23 20:35:48  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIFORMSELECTCHILD_H
#define	INC_RVC_UIFORMSELECTCHILD_H

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef	INC_RVC_OBJECT_H
#include <rvc/object.h>
#endif


namespace RVC {
namespace UI {
	
//===================================================================================================================
//! Combobox that contains the list of children from a given parent to select from given a possible filter.
//! No predefined items (such as "none") will be automatically added.  Use the AddItem method to add non-child
//! choices to the combobox.
class FORM_SELECT_CHILD : public MGUI::FORM_COMBOBOX {
	public:

		//! Constructor.
		FORM_SELECT_CHILD ();

		//! Destructor.
		virtual ~FORM_SELECT_CHILD (
			);

		//! Add list of children for specified parent OBJECT.
		//! Create must be called before using this method.
		//! Any previous children will be automatically removed prior to adding the new child list at the end.
		//! Other items may be freely added or removed from the combobox using AddItem without the need to avoid conflicting IDs.
		//! However, if using DeleteAllItems, DeleteChildren should be called first to properly update.
		//! When determining which item is selected, first use GetSelectedChild and if the returned name is empty then
		//! GetSelectedItemID may be used to determine which non-child selection was used.  The ID used for child items will be INT32_MIN.
		void AddChildren (
			const RVC::OBJECT& ParentObj,					//!< Parent object
			RVC::OBJTYPE ObjTypeChild,						//!< OBJTYPE of children to add
			bool DftLastUsed,									//!< Default to last-used child of specified type
			bool resize = true,								//!< Resize to content width
			RVC::OBJECT::FILTER *pFilter = 0				//!< Optional filter to limit choices
			);

		//! Add list of children for specified parent OBJITEM.
		//! See documention for other AddChildren method for additional details.
		void AddChildren (
			const RVC::OBJITEM& ObjItemParent,			//!< OBJITEM of parent
			RVC::OBJTYPE ObjTypeChild,						//!< OBJTYPE of children to add
			bool DftLastUsed,									//!< Default to last-used child of specified type
			bool resize = true,								//!< Resize to content width
			RVC::OBJECT::FILTER *pFilter = 0				//!< Optional filter to limit choices
			);

		//! Delete all children from combobox.
		void DeleteChildren (
			);

		//! Get the short name of the currectly selected child object.
		//! If a non-child choice is selected, the OBJECTNAME returned will be empty.
		const RVC::OBJECTNAME& GetSelectedChildName (
			) const;

		//! Determine if child object is currently selected.
		bool HasSelectedChild () const;

		//! Set the selected combobox item via the child's short name.
		//! Has no effect if Create() has not been called.
		//! @return true if match found, false if not.
		bool SetSelectedChildName (
			const RVC::OBJECTNAME& ChildName,			//!< Short name of child to select, has no effect if no match found
			bool notify = false								//!< Notify via delegate
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SIMPLE_ARRAY<RVC::OBJECTNAME> m_ObjectNames;
		int m_IdxFirstChild;
		
		FORM_SELECT_CHILD (const FORM_SELECT_CHILD&);
		FORM_SELECT_CHILD& operator= (const FORM_SELECT_CHILD&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//===================================================================================================================
}		// End namespace UI
}		// End namespace RVC

#endif	// INC_RVC_UIFORMSELECTCHILD_H
