/******************************************************************************
 *
 * \file sml/geostruct.h
 * \brief SML GEOSTRUCT
 *
 * \if NODOC
 * $Log: geostruct.h_v $
 * Revision 1.3  2011/10/12 14:39:20  mju
 * Remove redundant hdr.
 *
 * Revision 1.2  2007/05/08 15:45:58  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/04 21:44:37  dwilliss
 * Initial revision
 *
 * Revision 1.1  2007/05/04 21:35:07  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_GEOSTRUCT_H
#define INC_SML_GEOSTRUCT_H

#ifndef INC_MI32_SPATREF_H
#include <mi32/spatref.h>
#endif


#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif


/*****************************************************************************/
#ifndef GENERATING_DOXYGEN_OUTPUT

class TRANS2D_AFFINE;

namespace SML {
//! XXX As of 6.8 cuttoff, this structure should be changed to just contain
//! a TRANS2D_MAPGEN and two flags.  The bIsTempMem can go away, because they
//! should be allocated with new/delete.  In fact, I might be able to just
//! switch to using a TRANS2D_MAPGEN.  If I use SML::AUTOCLASS_NEW_T, it will
//! have a copy constructor and it should always get freed
//! ... Although this is basically half of a TRANS2D_MAPGEN
class RVCSYSLIBCLASSEXPORT GEOSTRUCT {
	public:

	GEOSTRUCT();
	~GEOSTRUCT();
	GEOSTRUCT(const GEOSTRUCT&);

	const SPATREF::COORDREFSYS& GetCoordRefSys (
		) const;

	void SetCoordRefSys (
		const SPATREF::COORDREFSYS& refsys
		);

	void GetTransAffine (
		TRANS2D_AFFINE& affine
		) const;

	const MAT3X3& GetTransOtoM (
		) const;

	const MAT3X3& GetTransMtoO (
		) const;

	void SetTransAffine (
		const MAT3X3& OtoM,
		const MAT3X3& MtoO
		);

	void SetTransAffine (
		const TRANS2D_AFFINE& affine
		);

	private:

		SPATREF::COORDREFSYS m_refsys;
		MAT3X3 m_OtoM, m_MtoO;

	};
#endif // GENERATING_DOXYGEN_OUTPUT

} // end of SML namespace

typedef SML::GEOSTRUCT SMLGEOSTRUCT;

#undef RVCSYSLIBEXPORT
#undef RVCSYSLIBCLASSEXPORT

#endif		//  INC_SML_GEOSTRUCT_H


