/**
 * \file soa/arcimsuilayerselect.h
 * \brief SOA ARCIMS User Interface for layer selection
 *
 * \if NODOC
 * $Id: arcimsuilayerselect.h_v 1.4 2008/09/02 21:45:05 mju Exp $
 *
 * $Log: arcimsuilayerselect.h_v $
 * Revision 1.4  2008/09/02 21:45:05  mju
 * Fix doxygen warning.
 *
 * Revision 1.2  2007/09/14 22:31:10  dwilliss
 * removed style stuff since ArcIMS doesn't use it
 *
 * Revision 1.1  2007/09/14 22:07:22  dwilliss
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SOA_ARCIMSUILAYERSELECT_H
#define	INC_SOA_ARCIMSUILAYERSELECT_H

#ifndef	INC_MGUI_FORM_H
	#include <mgui/form.h>
#endif

class MISTRINGLIST;

namespace SOA {
namespace ARCIMS {

class CONNECTION;

//=====================================================================================================================

class UI_FORM_LAYERSELECT : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		UI_FORM_LAYERSELECT ();

		//! Destructor.
		virtual ~UI_FORM_LAYERSELECT ();

		//! Create form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane
			);

		//! Get list of selected layer ids.
		const MISTRINGLIST& GetSelectedLayerIDs (
			) const;

		//! Get list of selected layer names.
		const MISTRINGLIST& GetSelectedLayerNames (
			) const;

		//! Set delegate to call when layer selection changed.
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set connection.
		//! If successful (and changed) will reset selected layer state to default.
		ERRVALUE SetConnection (
			const ARCIMS::CONNECTION& Connection,
			bool notify
			);

		//! Set selected layer names and styles.
		void SetSelection (
			const MISTRINGLIST& LayerNameList,
			bool notify
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		UI_FORM_LAYERSELECT (const UI_FORM_LAYERSELECT&);
		UI_FORM_LAYERSELECT& operator= (const UI_FORM_LAYERSELECT&);
		#endif
	};

//=====================================================================================================================

}	// End namespace ARCIMS
}	// End namespace SOA

#endif	// INC_SOA_ARCIMSUILAYERSELECT_H
