/**
 * \file imagefilterdeindex.h <spatmod/imagefilterdeindex.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_DEINDEX definition.
 *
 * \if NODOC
 * $Id: imagefilterdeindex.h_v 1.2 2006/01/30 17:40:16 mju Exp $
 *
 * $Log: imagefilterdeindex.h_v $
 * Revision 1.2  2006/01/30 17:40:16  mju
 * Rename sample iterator class.
 *
 * Revision 1.1  2005/08/16 15:09:33  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERDEINDEX_H
#define	INC_SPATMOD_IMAGEFILTERDEINDEX_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGEBUFFER_H
#include <spatmod/imagebuffer.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to de-index image, changing from index into palette to separate samples.
class FILTER_DEINDEX : public FILTER {
	public:

		//! Construct with same target data type and mode for all samples and default range.
		FILTER_DEINDEX (
			STAGE *pSourceImage,						//!< Source image stage
			bool TakeOwnership						//!< Filter takes ownership of source stage and will delete when needed
			);

		// Destructor.
		virtual ~FILTER_DEINDEX ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		SAMPLEITERATOR *m_itTgt;
		const PIXELTABLE *m_pPixelTable;
		UINT32 m_NumTableItems;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_DEINDEX (const FILTER_DEINDEX&);
		FILTER_DEINDEX& operator= (const FILTER_DEINDEX&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERDEINDEX_H

