/**
 * \file imagefilterlimitrange.h <spatmod/imagefilterlimitrange.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_LIMIT_RANGE definition.
 *
 * \if NODOC
 *
 * $Id: imagefilterlimitrange.h_v 1.1 2008/12/17 21:14:34 mju Exp $
 *
 * $Log: imagefilterlimitrange.h_v $
 * Revision 1.1  2008/12/17 21:14:34  mju
 * Initial revision
 *
 * Revision 1.1  2006/12/11 15:57:53  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERLIMITRANGE_H
#define	INC_SPATMOD_IMAGEFILTERLIMITRANGE_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to limit pixel to range.
//! All ranges are inclusive - values equal to range limits are included in range.
class FILTER_LIMIT_RANGE : public FILTER {
	public:

		//! Constructor using single range of PIXEL values to test for.
		FILTER_LIMIT_RANGE (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const PIXEL& MinValue,					//!< Minimum value (inclusive)
			const PIXEL& MaxValue,					//!< Maximum value (inclusive)
			bool invalidate = false						//!< True to output 1 if invalidate, otherwise output is 1 if inside
			);

		//! Constructor using single range of 'double' values to test for.
		//! Input image must have single component.
		FILTER_LIMIT_RANGE (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			double MinValue,							//!< Minimum value (inclusive)
			double MaxValue,							//!< Maximum value (inclusive)
			bool invalidate = false						//!< True to output 1 if invalidate, otherwise output is 1 if inside
			);

		//! Destructor.
		virtual ~FILTER_LIMIT_RANGE ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		PIXEL m_PixelMin;
		PIXEL m_PixelMax;
		bool m_Invalidate;

		void ProcessUINT8 (UINT16 SampleIdx, UINT8 *pSample, UINT8 *pValidity, UINT32 NumColumns) const;
		void ProcessSINT8 (UINT16 SampleIdx, INT8 *pSample, UINT8 *pValidity, UINT32 NumColumns) const;
		void ProcessUINT16 (UINT16 SampleIdx, UINT16 *pSample, UINT8 *pValidity, UINT32 NumColumns) const;
		void ProcessSINT16 (UINT16 SampleIdx, INT16 *pSample, UINT8 *pValidity, UINT32 NumColumns) const;
		void ProcessUINT32 (UINT16 SampleIdx, UINT32 *pSample, UINT8 *pValidity, UINT32 NumColumns) const;
		void ProcessSINT32 (UINT16 SampleIdx, INT32 *pSample, UINT8 *pValidity, UINT32 NumColumns) const;
		void ProcessFloat (UINT16 SampleIdx, float *pSample, UINT8 *pValidity, UINT32 NumColumns) const;
		void ProcessDouble (UINT16 SampleIdx, double *pSample, UINT8 *pValidity, UINT32 NumColumns) const;

		// STAGE overrides.
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual ERRVALUE v_Initialize ();

		// Unimplemented
		FILTER_LIMIT_RANGE (const FILTER_LIMIT_RANGE&);
		FILTER_LIMIT_RANGE& operator= (const FILTER_LIMIT_RANGE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERLIMITRANGE_H

